###################################################################
#
# BestSequencesPicker.r
#
# Load MDS data given chosen parameters, generate clusters, 
# pick clusters, and finally produce a list of neurons that best cluster.
# The list can be based on cluster only, or on cluster and metadata
# group The list can also be generated using z-score 
# distance cutoff for a given cluster model.
#
###################################################################

############# *SET UP ################

systemType <- "PC"
systemType <- "KrasClust"
arborNames <- c("Axon","Apical","Dendrite")
arborTypes <- c("Axon","ApicalDendrite","Dendrite")

# Set up directories
if (systemType == "PC"){
  clusterDir <- "C:/Documents and Settings/gillette/My Documents/research/ClusterAnalysis/"
  baseCodeDir <- "G:/Dropbox/RCode/"
  clusterGlobalDir <- "C:/Documents and Settings/gillette/My Documents/research/GlobalAlignment/"
  globalAlignDataDir <- "G:/AlignmentData/"
  domainDir <- "C:/Documents and Settings/gillette/My Documents/research/MultipleAlignment and Domains/"
}else{
  userDir <- "/home/tgillett/"
  clusterDir <- "/home/tgillett/ClusterAnalysis/"
  baseCodeDir <- "/home/tgillett/RUtils/"
  clusterGlobalDir <- "/home/tgillett/PairwiseAlignment/"
  globalAlignDataDir <- clusterGlobalDir
  baselineDir <- "/home/tgillett/PairwiseAlignment/Baseline/"
  domainDir <- "/home/tgillett/Domains/"
}

# Load Utils
source(paste(baseCodeDir,"Utils.r",sep=""))
source(paste(baseCodeDir,"StringUtils.r",sep=""))
source(paste(baseCodeDir,"StatisticsUtils.r",sep=""))

# Get metadata (in ClassCOmparisonDefs.r) and load related functions
source(paste(clusterDir,"ClusterStatistics.r",sep=""))
source(paste(clusterDir,"ClusterAnalysisFunctions.r",sep=""))
source(paste(clusterDir,"ClassComparisonDefs.r",sep=""))
clusterMetaData <- usedMetaData
sequenceNamesLists <- list()
for (i in 1:length(arborNames)){
  sequenceNamesLists[[arborNames[i]]] <- rownames(subset(clusterMetaData,ArborType==arborTypes[i]))
}

#library(parallel)
require(MASS)
require(mclust)

mycolors <- c("black","red","green","blue","darkorange","purple1","cyan","brown","magenta","green4","yellow2","pink1","gray70","aquamarine","purple4","navy")
mymodels <- c("EII","VII","EEI","VEI","EEE")
#mymodels <- c("EII","VII","EEI","VEI","EVI","EEE","VVI","VVV")

emResults <- list()

###### *Specific Run Settings ########

# Read correlation matrix
travOrder <- "STL"
# If this is a random dataset rather than NMO
isRandom <- F
#baseline <- 10
randStr <- ""
spurLen <- 0
#spurLen <- 5
spurStr <- ""
#spurStr <- paste("_SpurLen",spurLen,sep="")
scoreParamStr <- "G3E1Identity"
#scoreParamStr <- "G2E1Identity"
#scoreParamStr <- "G1E1Identity"
#scoreParamStr <- "G0E1Identity"
#matrixExt <- ".csv"
matrixExt <- ".tab"
rownamesVal <- 1 # Could be NULL

if (isRandom){
  randStr <- paste("Rand",baseline,sep="")
}
if (spurLen > 0){
  spurRemovedLengths <- read.csv(paste("SequenceLengths-NM",spurStr,"-MapType2.csv",sep=""),row.names=1)
  clusterMetaData <- spurRemovedLengths[rownames(clusterMetaData),"SeqLen"]
}else{
  clusterMetaData <- usedMetaData
}

###### Load MDS Data #####

# Values taken from ClusterAnalysisAllData.r, determined via experimentation

numDimsInitList <- list()

numDimsInitList[["G3E1Identity-STL-Apical"]] <- 8
numDimsInitList[["G3E1Identity_SpurLen5-STL-Apical"]] <- 8
numDimsInitList[["G2E1Identity-STL-Apical"]] <- 8
numDimsInitList[["G1E1Identity-STL-Apical"]] <- 7
numDimsInitList[["G3E1Identity-LTS-Apical"]] <- 8
numDimsInitList[["G3E1Identity-Both-Apical"]] <- 8

numDimsInitList[["G3E1Identity-STL-Axon"]] <- 7
numDimsInitList[["G3E1Identity_SpurLen5-STL-Axon"]] <- 7
numDimsInitList[["G2E1Identity-STL-Axon"]] <- 7
numDimsInitList[["G1E1Identity-STL-Axon"]] <- 7
numDimsInitList[["G3E1Identity-LTS-Axon"]] <- 9
numDimsInitList[["G3E1Identity-Both-Axon"]] <- 8

numDimsInitList[["G3E1Identity-STL-Dendrite"]] <- 7
numDimsInitList[["G3E1Identity_SpurLen5-STL-Dendrite"]] <- 7
numDimsInitList[["G2E1Identity-STL-Dendrite"]] <- 7
numDimsInitList[["G1E1Identity-STL-Dendrite"]] <- 7
numDimsInitList[["G3E1Identity-LTS-Dendrite"]] <- 8 # need to run
numDimsInitList[["G3E1Identity-Both-Dendrite"]] <- 8 # need to run

numDimsList <- list()

numDimsList[["G3E1Identity-STL-Apical"]] <- 4
numDimsList[["G3E1Identity_SpurLen5-STL-Apical"]] <- 4 #Unconfirmed
numDimsList[["G2E1Identity-STL-Apical"]] <- 4 #Unconfirmed
numDimsList[["G1E1Identity-STL-Apical"]] <- 4 #Unconfirmed
#numDimsList[["G3E1Identity-LTS-Apical"]] <- 8
#numDimsList[["G3E1Identity-Both-Apical"]] <- 8

numDimsList[["G3E1Identity-STL-Axon"]] <- 2
numDimsList[["G3E1Identity_SpurLen5-STL-Axon"]] <- 3 #Unconfirmed
numDimsList[["G2E1Identity-STL-Axon"]] <- 3 #Unconfirmed
numDimsList[["G1E1Identity-STL-Axon"]] <- 3 #Unconfirmed
#numDimsList[["G3E1Identity-LTS-Axon"]] <- 9
#numDimsList[["G3E1Identity-Both-Axon"]] <- 8

numDimsList[["G3E1Identity-STL-Dendrite"]] <- 4
numDimsList[["G3E1Identity_SpurLen5-STL-Dendrite"]] <- 4 #Unconfirmed
numDimsList[["G2E1Identity-STL-Dendrite"]] <- 4 #Unconfirmed
numDimsList[["G1E1Identity-STL-Dendrite"]] <- 4 #Unconfirmed
#arborName <- "Dendrite"
#arborName <- "Axon"
#arborName <- "Apical"

arborMDS <- list()
distMats <- list()
for (arborName in arborNames){
  # Set numDims for each arbor type assuming scoring, spur, and traversal parameters
  key <- paste(scoreParamStr,spurStr,"-",travOrder,"-",arborName,sep="")
  numDimsList[[arborName]] <- numDimsList[[key]]
  numDimsInitList[[arborName]] <- numDimsInitList[[key]]
  arborMDS[[arborName]] <- read.csv(paste(globalAlignDataDir,arborName,"_",travOrder,spurStr,"_",scoreParamStr,"_MDS",numDimsInitList[[arborName]],".csv",sep=""),row.names=1)
  distMats[[arborName]] <- as.matrix(read.csv(paste(globalAlignDataDir,arborName,"Distances_",travOrder,spurStr,"_",scoreParamStr,".csv",sep=""),row.names=1,check.names=F))
}

specificationsByArbor <- list(AxonMajor=AllAxonMajorSpecifications,Axon=AllAxonSpecifications,ApicalNC=AllApicalSpecificationsByHPNC,Apical=AllApicalSpecificationsByLayer,Dendrite=AllDendriteSpecifications)

#### Select an arbor type and specification and run cluster analysis ####

#! For axons with 3of7 dimensions, there will be 1 cluster. Setting 4 clusters    #
#  produces the result used on the poster, and that should be used in the paper.  #
#  Using 2of7 dimensions, the resulting number of clusters is 3. With 2of7        #
#  dimensions and only using specified neurites, the number of clusters is 4.    !#

with(
  subset(usedMetaData,ArborType=="ApicalDendrite"&Region2=="CA1"&Class2=="Pyramidal cell"&ExperCond=="Control"),
  aggregate(Class3,list(Species,Archive),length))
#  aggregate(Class3,list(Species,AgeClass,Protocol,Strain,Gender),length))

## Set parameters for rest of run ##
# In standard deviations of ?; 0 means no limit, use EM classification
distanceLimit <- 1
#distanceLimit <- 0
limitBySpecification <- NULL

extractFromZoom <- T
#arborName <- "Apical"

arborNames <- c("AxonMajor","Axon","ApicalNC","Apical","Dendrite")
arborTypes <- c("Axon","Axon","Apical","Apical","Dendrite")

#for (arborName in arborNames){
for (arborInd in 1:length(arborNames)){
  arborName <- arborNames[arborInd]
  arborType <- arborTypes[arborInd]
#arborType <- arborName

featMat <- arborMDS[[arborType]]
numDimsInit <- ncol(featMat)
numDims <- numDimsList[[arborType]]

sharedSeqs <- intersect(rownames(featMat),rownames(distMats[[arborType]]))
featMat <- featMat[sharedSeqs,]

specifications <- specificationsByArbor[[arborName]]

# Eliminate culture and non-control neurites
allowedSeqs <- rownames(featMat)[usedMetaData[rownames(featMat),"Protocol"]!="Culture"&usedMetaData[rownames(featMat),"ExperCond"]=="Control"]
featMat <- featMat[allowedSeqs,]

setG <- NA
setGStr <- ""
if (!is.na(setG)){
  setGStr <- paste("_",setG,sep="")
}
specifiedNeurons <- F
specifiedNeuronsStr <- ""
if (specifiedNeurons){
  specifiedNeuronsStr <- "_SpecNeurons"
}
#specifiedModel <- "EII"
specifiedModel <- "VEV"
#numDims <- numDimsInit
#numDims <- 5

# Set the relevant groups and get rid of feature vectors associated with unrepresented neurites
subsetObj <- loadSubsetSeqs(specifications,clusterMetaData,candidates=rownames(featMat))
if (specifiedNeurons){
  featMat <- featMat[subsetObj$Sequences,]
}
groupIds <- subsetObj$GroupIds
numGroups <- length(specifications)
#AllGroupIdsByFeatMat <- rep(0,nrow(featMat))


if (numDims == numDimsInit){
  key <- paste(travOrder,arborName,spurStr,scoreParamStr,specifiedNeuronsStr,specifiedModel,numDimsInit)
}else{
  key <- paste(travOrder,arborName,spurStr,scoreParamStr,specifiedNeuronsStr,specifiedModel,numDimsInit,numDims)
}
if (!is.na(setG)){
  key <- paste(key," G",setG,sep="")
}
key <- gsub(" +"," ",key)

# Run EM (picking best model)
if (key %in% names(emResults)){
  featEM <- emResults[[key]]
}else{
  if (is.na(setG)){
    if (specifiedModel == ""){
      featEM <- Mclust(featMat[,1:numDims],G=1:(numGroups+2))
    }else{
      featEM <- Mclust(featMat[,1:numDims],G=1:(numGroups+2),modelNames=specifiedModel)
    }
  }else{
    if (specifiedModel == ""){
      featEM <- Mclust(featMat[,1:numDims],G=setG)
    }else{
      featEM <- Mclust(featMat[,1:numDims],G=setG,modelNames=specifiedModel)
    }
  }
  emResults[[key]] <- featEM
}

groupNames <- paste(names(specifications)," (",sapply(subsetObj$SeqsByGroup,length),")",sep="")
clusterAssocStats <- clusteringStatisticalAnalysis(featMat,featEM,subsetObj,groupNames)
filenameBase <- paste(clusterDir,"/Subresults/",arborName,travOrder,spurStr,"_",scoreParamStr,specifiedNeuronsStr,"_MDS",numDims,"of",numDimsInit,"G",featEM$G,specifiedModel,sep="")
write.csv(round(clusterAssocStats$LogP,2),paste(filenameBase,"_LogPVals",".csv",sep=""))

png(paste("/home/tgillett/ClusterAnalysis/ParameterTesting/ClusterClass_",arborName,spurStr,scoreParamStr,"_",numDims,"of",numDimsInit,"Dims",specifiedNeuronsStr,setGStr,specifiedModel,".png",sep=""))
plot(featEM,what="classification",dimens=1:2)
dev.off()

png(paste("/home/tgillett/ClusterAnalysis/ParameterTesting/ClusterClass_",arborName,spurStr,scoreParamStr,"_",numDims,"of",numDimsInit,"Dims",specifiedNeuronsStr,setGStr,specifiedModel,"-ViewDims34.png",sep=""))
plot(featEM,what="classification",dimens=3:4)
dev.off()

png(paste("/home/tgillett/ClusterAnalysis/ParameterTesting/ClusterClass_",arborName,spurStr,scoreParamStr,"_",numDims,"of",numDimsInit,"Dims",specifiedNeuronsStr,setGStr,specifiedModel,"-AllDims.png",sep=""))
plot(featEM,what="classification")
dev.off()

png(paste("/home/tgillett/ClusterAnalysis/ParameterTesting/ClusterBIC_",arborName,spurStr,scoreParamStr,"_",numDims,"of",numDimsInit,"Dims",specifiedNeuronsStr,setGStr,".png",sep=""))
plot(featEM,what="BIC")
dev.off()

#require(car)
#plot(featEM[,1:2],asp=1)
#ellipse(parms$mean[1:2,1],shape=parms$variance$sigma[1:2,1:2,1],radius=1,center.pch=as.character(as.hexmode(1)),col=1)
#
#assocStats <- clusteringStatisticalAnalysis(featMat,featEM,subsetObj,names(specifications))

##### Run MDS on distance data of selected cluster #####
# Provide specificaiton
#limitBySpecification <- AllAxonSpecifications[[8]]
#specName <- "UPNs"
#limitBySpecification <- AllApicalSpecificationsByLayer[[1]]
#specName <- "CA1"
#limitBySpecification <- AllApicalSpecificationsByLayer[[2]]
#specName <- "CA3"

extractionOnly <- T
#clusterNum <- 3
for (clusterNum in 1:featEM$G){

if (numDims != numDimsInit){
  filename <- paste(domainDir,"SeqList-",arborName,spurStr,"-",scoreParamStr,"-Dims",numDims,"of",numDimsInit,specifiedNeuronsStr,"-",featEM$modelName,featEM$G,"-Clust",clusterNum,sep="")
}else{
  filename <- paste(domainDir,"SeqList-",arborName,spurStr,"-",scoreParamStr,"-Dims",numDims,specifiedNeuronsStr,"-",featEM$modelName,featEM$G,"-Clust",clusterNum,sep="")
}

if (extractFromZoom){
  # Select sequences from the cluster
  sequencesInCluster <- rownames(featMat)[clusterNum==featEM$classification]
  if (!is.null(limitBySpecification)){
    sequencesInCluster <- intersect(sequencesInCluster,loadSubsetSeqs(list(limitBySpecification),clusterMetaData)$Sequences)
  }
  distMat <- distMats[[arborType]][sequencesInCluster,sequencesInCluster]
  
  # Run MDS
  clusterMDS <- isoMDS(distMat,k=numDimsInit)
  clusterFeatMat <- clusterMDS$points[,1:numDims]
  filename <- paste(filename,"-Zoom",sep="")

  means <- apply(clusterFeatMat,2,mean)
  stdevs <- apply(clusterFeatMat,2,sd)
  numDimsAfter <- numDims
  
  # For digging in, should comment out when simply extracting
  if (!extractionOnly){
    clustSubsetObj <- loadSubsetSeqs(specifications,clusterMetaData,candidates=sequencesInCluster)
    allGroupIds <- rep(length(specifications)+1,length(sequencesInCluster)) # Includes non-specified neurons by adding an extra group "All Others"
    names(allGroupIds) <- sequencesInCluster
    allGroupIds[clustSubsetObj$Sequences] <- clustSubsetObj$GroupIds
    clustSubsetObj$AllGroupIds <- allGroupIds
    plot(clusterFeatMat[,1:2],col=allGroupIds)
  }
}else{
  numDimsAfter <- numDims
  clusterFeatMat <- featMat
  
  # Get model parameters for distance calculations
  params <- featEM$parameters
  means <- params$mean[,clusterNum]
  if (substr(featEM$modelName,1,1) == "E"){
    if (substr(featEM$modelName,3,3) %in% c("I","E")){
      stdevs <- rep(sqrt(params$variance$sigmasq),featEM$d)
    }else{
      stdevs <- sqrt(params$variance$scale*params$variance$shape)
    }
  }else{
    stdevs <- sqrt(params$variance$scale[clusterNum]*params$variance$shape)
  }
}

# Calculate distances (in std devs) in each dimension from model center (mean)
dimDistances <- t(apply(clusterFeatMat[,1:numDimsAfter],1,function(x,means,stdevs){
  return((x-means)/stdevs)
},means,stdevs))
finalDistSq <- apply(dimDistances,1,function(x){sum(x^2)})

if (distanceLimit == 0){
  # No distance limit, so use EM classification
  selectedSequences <- subsetObj$Sequences[featEM$classification==clusterNum]
}else{
  filename <- paste(filename,"-DistLim",distanceLimit,sep="")
  
  # Calculate distance limit using given parameter and chisq distribution
  realDistLim <- qchisq(1-2*pnorm(-distanceLimit),df=numDimsAfter)
  selectedSequences <- rownames(clusterFeatMat)[finalDistSq <= realDistLim]
}

# Take subset of selected sequences that fit specification (if given)
if (!is.null(limitBySpecification)){
  selectedSequences <- intersect(selectedSequences,loadSubsetSeqs(list(limitBySpecification),clusterMetaData)$Sequences)
  filename <- paste(filename,"-",specName,sep="")
}
# Complete filename
filename <- paste(filename,".txt",sep="")

# Sort sequences by nearness to center
selectedSequences <- names(sort(finalDistSq[selectedSequences]))
cor(finalDistSq[selectedSequences],usedMetaData[selectedSequences,"SeqLen"])

# Write to file
write(selectedSequences,filename)
}

}

#### Relationship between position and size ####
sizes <- clusterMetaData[rownames(featMat),"SeqLen"]
#ptSizes <- log(sizes)-log(median(sizes))
#ptSizes <- sizes/median(sizes)
ptSizes <- sqrt(sizes/median(sizes))
plot(featMat[,1],featMat[,2],col=groupIds,cex=ptSizes)
plot(featMat[,1],featMat[,2],col=featEM$classification,cex=ptSizes)