###########################################################################
#
# ClusterAnalysis.r (was ClusterAnalysisV4.r)
#    This version focuses on clustering subtypes defined by specifications.
#
#    NMers are combined across n, from 1 to 3, and across traversal orders
#    for n=3. Principal components should eliminate 3-mers that are 
#    equivalent across traversal orders.
#
#    Ngrams have an n of 4, and are combined across traversal orders. Ngrams
#    that appear too infrequently should be downweighted by the process.
#    This may need to be confirmed in testing.
#
#    In one version of output, Rand Index values are normalized to the Rand 
#    Index of alignment score. In the case of clustering by some feature or 
#    PC with alignment score, normalization is based on the same feature or
#    PC without alignment score.
#
#
###########################################################################

systemType <- "PC"
systemType <- "Mac"
systemType <- "KrasClust"

# Set up directories
if (systemType == "PC"){
  cvDir <- "C:/Documents and Settings/gillette/My Documents/My Dropbox/cvGUI/"
  clusterDir <- "C:/Documents and Settings/gillette/My Documents/research/ClusterAnalysis/"
  baseCodeDir <- "G:/Dropbox/RCode/"
  motifAnalysisDir <- "C:/Documents and Settings/gillette/My Documents/My Dropbox/Motif Analysis/Motif Analysis R Files/"
  homeDir <- "C:/"
} else if (systemType =="Mac"){
  #cvDir <- "/Users/toddgillette/Dropbox/cvGUI/"
  #cvDataDir <- "/Users/toddgillette/Dropbox/cvGUI/"
}else{  
  cvDir <- "/home/tgillett/ClusterAnalysis/"
  cvDataDir <- "/home/tgillett/results/"
  clusterDir <- "/home/tgillett/ClusterAnalysis/"
  baseCodeDir <- "/home/tgillett/RUtils/"
  homeDir <- "/home/tgillett/"
}
#clusterDir <- "~/ClusterAnalysis/"

getPath <- function(filename){
  return(paste(clusterDir,filename,sep=""))
}

#source(paste(cvDir,"CV-functions.r",sep=""))

# Read in meta data
#masterMetaData <- loadMetaDataFile(getPath("arborMetaData-NM-MapType2.csv"),adjustRownames=F)
masterMetaData <- read.csv(paste(homeDir,"arborMetaData-NM-MapType2.csv",sep=""),na.strings="",stringsAsFactors=FALSE,row.names=1)
if ("CellType"%in%colnames(masterMetaData)){
  colnames(masterMetaData)[colnames(masterMetaData)=="CellType"] <- "Class2"
}

source(paste(motifAnalysisDir,"PRAnalysisFunctions.r",sep=""))
source(paste(baseCodeDir,"Utils.r",sep=""))
source(paste(baseCodeDir,"StringUtils.r",sep=""))
source(paste(baseCodeDir,"StatisticsUtils.r",sep=""))
source(paste(clusterDir,"SubclassExplorationFunctions.r",sep=""))
source(paste(clusterDir,"ClusterStatistics.r",sep=""))
source(paste(clusterDir,"ClusterAnalysisFunctions.r",sep=""))
source(paste(clusterDir,"ClassComparisonDefs.r",sep=""))

# Read in NS-BLAST distance matrix
# blastTravOrder <- "lts"
# blastTravOrderUC <- "LTS"
# gapParamString <- "G0E5"
# baselineMethod <- 5
# blastIsEVals <- F
# #nsDesc <- "Old NS-BLAST Norm Scores"
# distanceMatrixFile <- paste("distMatrix-blast-NM-neurite2",blastTravOrder,"-",gapParamString,"-baseline",baselineMethod,".csv",sep="")
# distanceMatrixFile <- "G:/cvGUI Data/distMatrix-map4-baseline4.csv"
# distanceMatrixFile <- "G:/cvGUI Data/distMatrix-NM-blast-noLength-map2-baseline4.csv"
# distanceMatrixFile <- "distMatrix-map4-baseline4.csv"
# distanceMatrixFile <- "distMatrix-NM-blast-noLength-map2-baseline4.csv"
# #distanceMatrixFile <- paste("G:/cvGUI Data/",distanceMatrixFilename,sep="")
# distanceMatrixFile <- getPath(distanceMatrixFilename)
# #distanceMatrixFile <- "/homes/tgillett/results/distMatrix-NM-blast-noLength-map4-baseline4.csv"
# #distanceMatrixFile <- "G:/cvGUI Data/distMatrix-NM-blast-noLength-map4-baseline4.csv"
# nsDistLTSDF <- read.csv(distanceMatrixFile,na.strings="",row.names=1)
# colnames(nsDistLTSDF) <- rownames(nsDistLTSDF)

distanceMatrixFilename <- paste(clusterDir,"alignmentDistSTL.TreeIdentityG3E1.csv",sep="")

distanceMatrixFilename <- paste("distMatrix-blast-NM-neurite2stl-",gapParamString,"-baseline",baselineMethod,".csv",sep="")
distanceMatrixFile <- getPath(distanceMatrixFilename)
nsDistSTLDF <- read.csv(distanceMatrixFilename,na.strings="",row.names=1,check.names=F)
#colnames(nsDistSTLDF) <- rownames(nsDistSTLDF)

# Read in morphometric feature vector data
#Morphometrics <- read.csv(getPath("SampleFeatureVector.csv"),row.names=1)
#Morphometrics <- read.csv(getPath("NM-Morphometrics_HWD.csv"),row.names=1)
Morphometrics <- read.csv(paste(clusterDir,"NM-Morphometrics_HWD.csv",sep=""),row.names=1)
# Remove some nonexisting sequences
Morphometrics <- Morphometrics[!rownames(Morphometrics)%in%removedSeqs,]
Morphometrics <- Morphometrics[apply(Morphometrics,1,function(x){sum(is.na(x))})==0,]
# If comparing to normalized scores from Global Alignment (ClusterAnalysisFromGlobalAlign.r)
#Morphometrics <- Morphometrics[colnames(normScoreMatDF),]

#Morphometrics <- Morphometrics[,!colnames(Morphometrics)%in%c("Width","Height","Depth","Stems")]
# Drop "Stems"; Drop "Bifurcations", since it's almost exactly the same as "Branches"
Morphometrics <- Morphometrics[,!colnames(Morphometrics)%in%c("Stems","Bifurcations")]
# Remove FractalDimension, which currently is incorrect
Morphometrics <- Morphometrics[,!colnames(Morphometrics)%in%c("FractalDimension")]

# If we're eliminating metrics that tend to varry by Archive
Morphometrics <- Morphometrics[,!colnames(Morphometrics)%in%c("Diameter","Contraction","LocalBifurcationAmpl","Fragmentation")]

# Read in nmer percentile rank data and create feature vectors from them
# Requires a run of some components in PRAnalysisFunctions.r - TODO get in format for sourcing
maxK <- 3
prDataDirectory <- "/homes/tgillett/results/PercentileRanks/"
#prDataDirectory <- "C:/Documents and Settings/gillette/My Documents/research/MotifAnalysis/PercentileRanks/"
PRVect <- read.csv(getPRFilename(1,sets=40,surrogateSet=2,order="LTS",method=8,threshold="0.0",thresholdTypeString="Whole",directory=prDataDirectory),stringsAsFactors=F,row.names=1)[-1]
for (k in 2:maxK){
  tmpPrVect <- read.csv(getPRFilename(k,sets=40,surrogateSet=2,order="LTS",method=8,threshold="0.0",thresholdTypeString="Whole",directory=prDataDirectory),stringsAsFactors=F,row.names=1)[-1]
  PRVect <- PRVect[rownames(tmpPrVect),]
  PRVect <- cbind(PRVect,tmpPrVect)
}
colnames(PRVect)[1] <- "C"
sourceTraversal <- "LTS"
tmpPrVect <- read.csv(getPRFilename(3,sets=40,surrogateSet=2,order="STL",method=8,threshold="0.0",thresholdTypeString="Whole",directory=prDataDirectory,sourceTraversal=sourceTraversal),stringsAsFactors=F,row.names=1)[-1]
colnames(tmpPrVect) <- paste("s",colnames(tmpPrVect),sep="")
PRVect <- PRVect[rownames(tmpPrVect),]
PRVect <- cbind(PRVect,tmpPrVect)
PRVect <- PRVect[,colnames(PRVect)!="sATT"&colnames(PRVect)!="ATT"]
# !grepl("s[ACT]C[ACT]",colnames(PRVect)) gets STL trimers that shouldn't overlap LTS trimers
PRVect <- PRVect[,!grepl("s[ACT]C[ACT]",colnames(PRVect))]

# Normalized kmer frequencies (kmer freq - mean baseline kmer freq) - might perform better than PRs
NFVect <- read.csv(getFreqDiffFilename(1,sets=40,surrogateSet=2,order="LTS",method=8,threshold="0.0",thresholdTypeString="Whole",directory=prDataDirectory,sourceTraversal="LTS"),stringsAsFactors=F,row.names=1)[-1]
for (k in 2:maxK){
  tmpNFVect <- read.csv(getFreqDiffFilename(k,sets=40,surrogateSet=2,order="LTS",method=8,threshold="0.0",thresholdTypeString="Whole",directory=prDataDirectory,sourceTraversal="LTS"),stringsAsFactors=F,row.names=1)[-1]
  NFVect <- NFVect[rownames(tmpNFVect),]
  NFVect <- cbind(NFVect,tmpNFVect)
}
colnames(NFVect)[1] <- "C"
sourceTraversal <- "STL"
tmpNFVect <- read.csv(getFreqDiffFilename(3,sets=40,surrogateSet=2,order="STL",method=8,threshold="0.0",thresholdTypeString="Whole",directory=prDataDirectory,sourceTraversal=sourceTraversal),stringsAsFactors=F,row.names=1)[-1]
colnames(tmpNFVect) <- paste("s",colnames(tmpNFVect),sep="")
NFVect <- NFVect[rownames(tmpPrVect),]
NFVect <- cbind(NFVect,tmpNFVect)
NFVect <- NFVect[,colnames(NFVect)!="sATT"&colnames(NFVect)!="ATT"]
# !grepl("s[ACT]C[ACT]",colnames(PRVect)) gets STL trimers that shouldn't overlap LTS trimers
NFVect <- NFVect[,!grepl("s[ACT]C[ACT]",colnames(NFVect))]

# NGrams (pure frequencies)
ngrams4LTS <- read.csv(paste(cvDir,"NGramFreqs_NM_M2_LTS_N4.csv",sep=""),row.names=1)
ngrams4STL <- read.csv(paste(cvDir,"NGramFreqs_NM_M2_STL_N4.csv",sep=""),row.names=1)
ngrams4STL <- ngrams4STL[,-grep("[ACT]ATT",colnames(ngrams4STL))]
colnames(ngrams4STL) <- paste("s",colnames(ngrams4STL),sep="")
distinct4Mers <- grep("s[ACT]+[AT][ACT]+",colnames(ngrams4LTS),value=T)
ngrams4 <- cbind(ngrams4LTS,ngrams4STL[distinct4Mers])

ngrams4Norm <- normalize(ngrams4)
ngrams4Centered <- centerFeatures(ngrams4Norm,FUN=median)


sharedSequences <- rownames(usedMetaData)
sharedSequences <- intersect(sharedSequences,rownames(nsDistSTLDF))
sharedSequences <- intersect(sharedSequences,rownames(Morphometrics))
sharedSequences <- intersect(sharedSequences,rownames(PRVect))
sharedSequences <- intersect(sharedSequences,rownames(NFVect))
sharedSequences <- intersect(sharedSequences,rownames(ngrams4))
#nsDistDF <- nsDistDF[sharedSequences,sharedSequences]
presentMetaData <- usedMetaData[sharedSequences,]
PRVect <- PRVect[sharedSequences,]
NFVect <- NFVect[sharedSequences,]
Morphometrics <- Morphometrics[sharedSequences,]
ArchiveMetrics <- ArchiveMetrics[sharedSequences,]
PCs <- prcomp(Morphometrics,scale.=T)$x
ngrams4Centered <- ngrams4Centered[sharedSequences,]
nsDistSTLDF <- nsDistSTLDF[sharedSequences,sharedSequences]

PRPrcomp <- prcomp(PRVect,scale.=T)

## 1. Transform data to get normally distributed data

# NMers: Convert from uniformly distributed to normally distributed
nmersXformed <- as.data.frame(apply(PRVect,2,qnorm))
# Fix 0 and 1 cases (-Inf and Inf)
nmersXformed <- as.data.frame(apply(nmersXformed,2,fixNmerTransform))

# Log transform any morphometrics that could benefit from it (details in function "logTransformData")
morphometricsXformed <- Morphometrics
morphometricsXformed$Depth[morphometricsXformed$Depth==0] <- 3
morphometricsXformed <- as.data.frame(apply(morphometricsXformed,2,logTransformData,threshold=.8))

archiveMetricsXformed <- ArchiveMetrics
archiveMetricsXformed <- as.data.frame(apply(archiveMetricsXformed,2,logTransformData,threshold=.8))

## 2. Scale the features (after transformation)
nmersXformed <- as.data.frame(scale(nmersXformed))
morphometricsXformed <- as.data.frame(scale(morphometricsXformed))

# We'll make sure the neurites are in the same order again after each clustering

# Get all well represented types
minSampleSize <- 150

#clusterTypes(specifications,typeMetaData,alignmentScoresLTS,alignmentScoresSTL,featureVectors,nmers,ngrams,method="ward",clusterMetric=AdjustedRandIndex,minGroupSize=30,numClusterRuns=20,sampleProportionally=T,sampleSize=if (sampleProportionally) 80 else minGroupSize,alignmentScoreWeight=2,maxCumulativePCs=6,debugFlag=1)
# specifications loaded from ClassComparisonDefs.r

#comparisonData <- list(ComparisonMeans=comparisonMeans,ComparisonSEMs=comparisonSEMs)
comparisonData <- list()
#for (index in 6:length(SpecificationLists)){
for (index in 1:length(SpecificationLists)){
  specifications <- SpecificationLists[[index]]
  specTitle <- names(SpecificationLists)[index]
  print(paste("Comparison ",index,": ",specTitle,sep=""))
  comparisonData[[index]] <- clusterTypes(specifications,typeMetaData=presentMetaData,alignmentScoresLTS=NA,alignmentScoresSTL=NA,featureVectors=Morphometrics,nmers=PRVect,ngrams=ngrams4Centered,method="ward",clusterMetric=AdjustedRandIndex,minGroupSize=34,numClusterRuns=20,sampleProportionally=F,sampleSize=34,alignmentScoreWeight=2,maxCumulativePCs=8,debugFlag=1)
  names(comparisonData)[index] <- specTitle
}
save.image("GrandCluster061812.RData")

# Write data to one file
stripIndices <- c()
for (index in 1:length(comparisonData)){
  if (length(comparisonData[[index]]) > 1){
    tmpData <- comparisonData[[index]]$ComparisonMeans
    tmpData <- tmpData[!grepl("Stems",names(tmpData))]
    if (index == 1){
      allCompDataDF <- as.data.frame(t(tmpData))
    }
    else{
      allCompDataDF <- rbind(allCompDataDF,as.data.frame(t(tmpData)))
    }
    tmpData <- comparisonData[[index]]$ComparisonSEMs
    tmpData <- tmpData[!grepl("Stems",names(tmpData))]
    allCompDataDF <- rbind(allCompDataDF,as.data.frame(t(tmpData)))
  }
  else{
    stripIndices <- append(stripIndices,index)
  }
}
#paste(c("Mean","SD"),sort(rep(1:length(comparisonData),2)))
goodComparisonNames <- names(comparisonData[-stripIndices])
rownames(allCompDataDF) <- paste(c("Mean","SD"),goodComparisonNames[sort(rep(1:length(comparisonData[-stripIndices]),2))])
#rep(c("Mean","SD"),length(comparisonData))

# Strip unexplored columns
allCompDataDF <- allCompDataDF[,apply(allCompDataDF,2,var)>0]
write.csv(allCompDataDF,"ClusterComparisons.csv")
save.image("GrandCluster061812.RData")

# Split best and worst comparisons
allCompDataMeans <- allCompDataDF[grepl("Mean",rownames(allCompDataDF)),]
#rownames(allCompDataMeans) <- goodComparisonNames
rownames(allCompDataMeans) <- sapply(rownames(allCompDataMeans),function(x){return(strsplit(x,"Mean ")[[1]][2])})
bestComparisons <- rownames(allCompDataMeans)[apply(allCompDataMeans,1,max)>.1]
worstComparisons <- rownames(allCompDataMeans)[apply(allCompDataMeans,1,max)<=.1]

#####################################
# Run comparisons focused on nmers

# Transform the data to look normally distributed
#PRVectNorm <- qnorm(PRVect)

minBifs <- 0
usePCs <- F
numClusterRuns <- 30
nmerFDComparisonData <- list()
nmerComparisonData <- list()
metricComparisonData <- list()
nmerRandComparisonData <- list()
metricRandComparisonData <- list()
#for (index in 6:length(SpecificationLists)){
for (index in 1:length(SpecificationLists)){
  specifications <- SpecificationLists[[index]]
  numGroups <- length(specifications)
  specTitle <- names(SpecificationLists)[index]
  #if (specTitle%in%bestComparisons){
  print(paste("Comparison ",index,": ",specTitle,sep=""))
  # Run pairwise comparisons
  for (i in 1:(numGroups-1)){
    for (j in (i+1):numGroups){
      specPair <- list(specifications[[i]],specifications[[j]])
      specITitle <- specifications[[i]][[length(specifications[[i]])]]
      if (length(specITitle) > 1){
        specITitle <- paste(specITitle,collapse="&")
      }
      specJTitle <- specifications[[j]][[length(specifications[[j]])]]
      if (length(specJTitle) > 1){
        specJTitle <- paste(specJTitle,collapse="&")
      }
      specPairTitle <- paste(specTitle,"-",specITitle,"vs",specJTitle)
      
      specPairGroups <- getSpecificationGroups(specPair,presentMetaData,minBifs=minBifs)
      sampleSize <- min(80,floor(0.75*min(sapply(specPairGroups,length))))
      
      print(paste("Comparison pair:",specPairTitle))
      
      # Running 30 subsamples
#       nmerComparisonData <- append(nmerComparisonData,
#                   list(clusterTypesSimple(specPair,typeMetaData=presentMetaData,nmersXformed,method="ward",clusterMetric=AdjustedRandIndex,minGroupSize=31,numClusterRuns=numClusterRuns,sampleProportionally=F,sampleSize=sampleSize,debugFlag=0,random=F,minBifs=minBifs,usePCs=usePCs)))
#       names(nmerComparisonData)[length(nmerComparisonData)] <- specPairTitle
#       
#       metricComparisonData <- append(metricComparisonData,
#                   list(clusterTypesSimple(specPair,typeMetaData=presentMetaData,morphometricsXformed,method="ward",clusterMetric=AdjustedRandIndex,minGroupSize=34,numClusterRuns=numClusterRuns,sampleProportionally=F,sampleSize=sampleSize,debugFlag=0,random=F,minBifs=minBifs,usePCs=usePCs)))
#       names(metricComparisonData)[length(metricComparisonData)] <- specPairTitle
      
      nmerFDComparisonData <- append(nmerFDComparisonData,
        list(clusterTypesSimple(specPair,typeMetaData=presentMetaData,NFVect,method="ward",clusterMetric=AdjustedRandIndex,minGroupSize=31,numClusterRuns=numClusterRuns,sampleProportionally=F,sampleSize=sampleSize,debugFlag=0,random=F,minBifs=minBifs)))
        #list(clusterTypesSimple(specPair,typeMetaData=presentMetaData,NFVect,method="ward",clusterMetric=AdjustedRandIndex,minGroupSize=31,numClusterRuns=numClusterRuns,sampleProportionally=F,sampleSize=sampleSize,debugFlag=0,random=F,minBifs=minBifs,usePCs=usePCs)))
      names(nmerFDComparisonData)[length(nmerFDComparisonData)] <- specPairTitle
      
    }
  }
  #}
}

#PairwiseSpecList run
# Eliminate poor performers from PairwiseSpecificationLists
PairwiseSpecificationLists <- PairwiseSpecificationLists[!names(PairwiseSpecificationLists)%in%worstPerformers]

minBifs <- 20
usePCs <- F
numClusterRuns <- 30
nmerFDComparisonData <- list()
nmerComparisonData <- list()
metricComparisonData <- list()
mergedComparisonData <- list()

metricClass <- "em"
metricClass <- "kmeans"
metricClass <- "hclust"

sampleSizes <- c()
for (index in 1:length(PairwiseSpecificationLists)){
  specifications <- PairwiseSpecificationLists[[index]]
  groups <- getSpecificationGroups(specifications,presentMetaData,minBifs=minBifs)
  sampleSizes[index] <- min(80,floor(0.75*min(sapply(groups,length))))
}
skipPairs <- c(79,80,83,75,16)
#results <- mclapply(PairwiseSpecificationLists[-skipPairs],clusterTypesSimple,typeMetaData=presentMetaData,featureVectors=morphometricsXformed,methodClass="kmeans",method="ward",clusterMetric=AdjustedRandIndex,minGroupSize=34,numClusterRuns=numClusterRuns,sampleProportionally=F,sampleSize=NA,debugFlag=0,random=F,minBifs=minBifs,usePCs=usePCs,mc.cores=10)
results <- mclapply(PairwiseSpecificationLists[allWorthRunning],clusterTypesSimple,typeMetaData=presentMetaData,featureVectors=morphometricsXformed,methodClass="kmeans",method="ward",clusterMetric=AdjustedRandIndex,minGroupSize=34,numClusterRuns=numClusterRuns,sampleProportionally=F,sampleSize=NA,debugFlag=0,random=F,minBifs=minBifs,usePCs=usePCs,mc.cores=10)
metricComparisonData <- results

results <- mclapply(PairwiseSpecificationLists[allWorthRunning],clusterTypesMerged,typeMetaData=presentMetaData,featureVectors=morphometricsXformed,alignmentDistMat=distMatDF,methodClass="kmeans",method="ward",clusterMetric=AdjustedRandIndex,minGroupSize=34,numClusterRuns=numClusterRuns,sampleProportionally=F,sampleSize=NA,debugFlag=0,random=F,minBifs=minBifs,usePCs=usePCs,mc.cores=10)
mergedComparisonData <- results

# for (index in 1:length(results)){
#   specifications <- PairwiseSpecificationLists[[index]]
#   numGroups <- length(specifications)
#   
#   metricComparisonData <- append(metricComparisonData,list(results[[index]]))
# }
# names(metricComparisonData) <- names(PairwiseSpecificationLists)

indicesToRun <- 1:length(PairwiseSpecificationLists)
# From ClusterAnalysisFromGlobalAlign.r
indicesToRun <- allWorthRunning


for (index in indicesToRun){
  specifications <- PairwiseSpecificationLists[[index]]
  numGroups <- length(specifications)
  specTitle <- names(PairwiseSpecificationLists)[index]
  #if (specTitle%in%bestComparisons){
  print(paste("Comparison ",index,": ",specTitle,sep=""))
  # Run pairwise comparisons
  groups <- getSpecificationGroups(specifications,presentMetaData,minBifs=minBifs)
  sampleSize <- min(80,floor(0.75*min(sapply(groups,length))))
      
  # Running 30 subsamples
  #       nmerComparisonData <- append(nmerComparisonData,
  #                   list(clusterTypesSimple(specifications,typeMetaData=presentMetaData,nmersXformed,method="ward",clusterMetric=AdjustedRandIndex,minGroupSize=31,numClusterRuns=numClusterRuns,sampleProportionally=F,sampleSize=sampleSize,debugFlag=0,random=F,minBifs=minBifs,usePCs=usePCs)))
  #       names(nmerComparisonData)[length(nmerComparisonData)] <- specPairTitle
  #       
#          metricComparisonData <- append(metricComparisonData,
#                      list(clusterTypesSimple(specifications,typeMetaData=presentMetaData,morphometricsXformed,methodClass="kmeans",method="ward",clusterMetric=AdjustedRandIndex,minGroupSize=34,numClusterRuns=numClusterRuns,sampleProportionally=F,sampleSize=sampleSize,debugFlag=0,random=F,minBifs=minBifs,usePCs=usePCs)))
#          names(metricComparisonData)[length(metricComparisonData)] <- specTitle
      
  mergedComparisonData <- append(mergedComparisonData,
                                 list(clusterTypesMerged(specifications,typeMetaData=presentMetaData,morphometricsXformed,distMatDF,methodClass="kmeans",method="ward",clusterMetric=AdjustedRandIndex,minGroupSize=34,numClusterRuns=numClusterRuns,sampleProportionally=F,sampleSize=sampleSize,debugFlag=0,random=F,minBifs=minBifs,usePCs=usePCs)))
  names(mergedComparisonData)[length(mergedComparisonData)] <- specTitle
  #nmerFDComparisonData <- append(nmerFDComparisonData,
  #                               list(clusterTypesSimple(specifications,typeMetaData=presentMetaData,NFVect,method="ward",clusterMetric=AdjustedRandIndex,minGroupSize=31,numClusterRuns=numClusterRuns,sampleProportionally=F,sampleSize=sampleSize,debugFlag=0,random=F,minBifs=minBifs,usePCs=usePCs)))
  #list(clusterTypesSimple(specPair,typeMetaData=presentMetaData,NFVect,method="ward",clusterMetric=AdjustedRandIndex,minGroupSize=31,numClusterRuns=numClusterRuns,sampleProportionally=F,sampleSize=sampleSize,debugFlag=0,random=F,minBifs=minBifs,usePCs=usePCs)))
  #names(nmerFDComparisonData)[length(nmerFDComparisonData)] <- specTitle
}

save.image("GrandCluster022113.RData")

# Write data to one file
stripIndices <- c()
nmerFDDataDF <- NULL
nmerDataDF <- NULL
metricDataDF <- NULL
mergedDataDF <- NULL
#for (index in 1:length(metricComparisonData)){
for (index in 1:length(mergedComparisonData)){
#for (index in 1:length(nmerFDComparisonData)){
  if (length(mergedComparisonData[[index]]) > 1){
  #if (length(metricComparisonData[[index]]) > 1){
  #if (length(nmerFDComparisonData[[index]]) > 1){
#    nmerFDMeansData <- nmerFDComparisonData[[index]]$ComparisonMeans
#    nmerMeansData <- nmerComparisonData[[index]]$ComparisonMeans
#    metricMeansData <- metricArchiveComparisonData[[index]]$ComparisonMeans
    #metricMeansData <- metricComparisonData[[index]]$ComparisonMeans
    mergedMeansData <- mergedComparisonData[[index]]$ComparisonMeans
    #if (is.null(nmerFDDataDF)){
    #if (is.null(metricDataDF)){
    if (is.null(mergedDataDF)){
    #  nmerFDDataDF <- as.data.frame(t(nmerFDMeansData))
    #if (is.null(nmerDataDF)){
      #nmerDataDF <- as.data.frame(t(nmerMeansData))
      #metricDataDF <- as.data.frame(t(metricMeansData))
      mergedDataDF <- as.data.frame(t(mergedMeansData))
    }else{
      #nmerFDDataDF <- rbind(nmerFDDataDF,as.data.frame(t(nmerFDMeansData)))
      #nmerDataDF <- rbind(nmerDataDF,as.data.frame(t(nmerMeansData)))
      #metricDataDF <- rbind(metricDataDF,as.data.frame(t(metricMeansData)))
      mergedDataDF <- rbind(mergedDataDF,as.data.frame(t(mergedMeansData)))
    }
  }
  else{
    stripIndices <- append(stripIndices,index)
  }
}

allCompDataDF <- cbind(nmerDataDF,metricDataDF)
rownames(allCompDataDF) <- names(nmerComparisonData[-stripIndices])
rownames(mergedDataDF) <- names(mergedComparisonData[-stripIndices])
rownames(metricDataDF) <- names(metricComparisonData[-stripIndices])
rownames(nmerFDDataDF) <- names(nmerFDComparisonData[-stripIndices])
allCompDataDF <- metricDataDF
allCompDataDF <- mergedDataDF
pairIndices <- sapply(rownames(allCompDataDF),function(x,listnames){return(which(x==listnames))},names(PairwiseSpecificationLists))
allCompDataDF <- cbind(allCompDataDF,PairIndex=pairIndices)

# Strip unexplored columns
allCompDataDF <- allCompDataDF[,apply(allCompDataDF,2,var)>0]
write.csv(allCompDataDF,"ClusterMetricComparisonsEM.csv")
write.csv(allCompDataDF,"ClusterMetricComparisonsWard.csv")
write.csv(allCompDataDF,"ClusterMetric&AlignmentMergedComparisonsKmeans.csv")
write.csv(allCompDataDF,"ClusterMetricComparisonsKmeans.csv")
write.csv(allCompDataDF,"ClusterNmerComparisons.csv")
write.csv(allCompDataDF,"Cluster Xformed Nmer&Metric Comparisons.csv")
write.csv(allCompDataDF,"Cluster Comparisons MinBifs22.csv")
write.csv(allCompDataDF,"Randomized Cluster Nmer & Metric Comparisons.csv")
write.csv(metricDataDF,"Cluster Archive-Metric Comparisons.csv")
write.csv(allCompDataDF,"Best Cluster Nmer & Metric PCs.csv")
write.csv(metricDataDF,"Best Cluster Metric Comparisons.csv")
write.csv(nmerFDDataDF,"Cluster Nmer FreqDiff Comparisons MinBifs22.csv")
write.csv(nmerFDDataDF,"Cluster Nmer FreqDiff Comparisons MinBifs0.csv")
#write.csv(allCompDataDF,"ClusterNmerComparisons_ArchFeatStripped.csv")

#####
# Some specific cases to visualize

png("CA1vsCA3Apical_BranchesvsBifAmpl.png")
plot(Morphometrics[groups[[1]],"Branches"],Morphometrics[groups[[1]],"RemoteBifurcationAmpl"],col=2)
points(Morphometrics[groups[[2]],"Branches"],Morphometrics[groups[[2]],"RemoteBifurcationAmpl"],col=3)
dev.off()

png("CA1vsCA3Apical_BOvsBifAmpl.png")
plot(Morphometrics[groups[[1]],"BranchOrder"],Morphometrics[groups[[1]],"RemoteBifurcationAmpl"],col=2)
points(Morphometrics[groups[[2]],"BranchOrder"],Morphometrics[groups[[2]],"RemoteBifurcationAmpl"],col=3)
dev.off()

png("CA1vsCA3Apical_BifAmplClust.png")
plot(1:length(subsample),Morphometrics[subsample,"RemoteBifurcationAmpl"],col=classPartition,pch=samplePartition)
dev.off()

########################################################
# Compare various result sets

# Compare FreqDiff and PercRank
freqDiffCompsMin22 <- read.csv("Cluster Nmer FreqDiff Comparisons MinBifs22.csv",row.names=1)
percRankAndMetricCompsMin22 <- read.csv("Cluster Comparisons MinBifs22.csv",row.names=1)
sharedSeqs <- intersect(rownames(freqDiffCompsMin22),rownames(percRankAndMetricCompsMin22))
freqDiffCompsMin22 <- freqDiffCompsMin22[sharedSeqs,]
percRankCompsMin22 <- percRankAndMetricCompsMin22[sharedSeqs,1:49]
metricCompsMin22 <- percRankAndMetricCompsMin22[sharedSeqs,50:62]

freqDiffCompsMin8 <- read.csv("Cluster Nmer FreqDiff Comparisons MinBifs0.csv",row.names=1)
percRankAndMetricCompsMin8 <- read.csv("Cluster Xformed Nmer&Metric Comparisons.csv",row.names=1)
sharedSeqs <- intersect(rownames(freqDiffCompsMin8),rownames(percRankAndMetricCompsMin8))
freqDiffCompsMin8 <- freqDiffCompsMin8[sharedSeqs,]
percRankCompsMin8 <- percRankAndMetricCompsMin8[sharedSeqs,1:49]
metricCompsMin8 <- percRankAndMetricCompsMin8[sharedSeqs,c(50:59,61:63)]

freqDiffCompsMin22Max <- apply(freqDiffCompsMin22,1,max)
percRankCompsMin22Max <- apply(percRankCompsMin22,1,max)
metricCompsMin22Max <- apply(metricCompsMin22,1,max)
goodPerformers <- percRankCompsMin22Max > .06 & freqDiffCompsMin22Max > .06
summary(percRankCompsMin22Max[goodPerformers]-freqDiffCompsMin22Max[goodPerformers])
summary(percRankCompsMin22Max-freqDiffCompsMin22Max)

compDataDFMeans <- allCompDataDF[grep("Mean ",rownames(allCompDataDF)),]
rownames(compDataDFMeans) <- names(nmerComparisonData[-stripIndices])
write.csv(compDataDFMeans,"ClusterNmerComparisons_Means.csv")
write.csv(compDataDFMeans,"ClusterNmerComparisons_Means_ArchFeatStripped.csv")

save.image("GrandCluster062812.RData")

## Look at confounds of given comparisons
comparisonName <- "ApicalDendrite/Neocortex/Young/Pyramidal cell by Order"
comparisonName <- "Dendrite/Motoneuron by BrainRegion" # Spinal cord vs Brainstem
comparisonName <- "ApicalDendrite/Rodent/Pyramidal cell by BrainRegion" # Hippocampus vs Anterior olfactory nucleus
comparisonName <- "ApicalDendrite/Rodent/Young/Pyramidal cell by BrainRegion" # Hippocampus vs Anterior olfactory nucleus
comparisonName <- "Dendrite/Primate/Neocortex/Layer 2,3/Pyramidal cell by CorticalRegion" # Prefrontal vs Temporal sulcus	
comparisonName <- "ApicalDendrite/Primate/Neocortex/Pyramidal cell by AgeClass" # Adult vs Young	
comparisonName <- "ApicalDendrite/Neocortex/Pyramidal cell by Order" # Rodent vs Elephant
comparisonName <- "Dendrite/Retina/Ganglion cell by Species" # Mouse vs Salamander
comparisonName <- "Axon/Rodent/Neocortex/Pyramidal cell by CorticalRegion" # Somatosensory vs Visual
comparisonName <- "ApicalDendrite/Primate/Neocortex/Prefrontal/Layer 2,3/Pyramidal cell by Species" # Monkey vs Human
comparisonName <- "Axon/Rodent/Neocortex/Pyramidal cell by Species" # Mouse vs Rat
comparisonName <- "ApicalDendrite/Neocortex/Adult/Pyramidal cell by Order" # Rodent vs Elephant

exploreSelectionConfounds(specificationList=SelectSpecificationLists[[comparisonName]],metaTypes=usedMetaCols,typeMetaData=usedMetaData,divergenceThreshold=-1)
exploreSelectionConfounds(specificationList=SpecificationLists[[comparisonName]],metaTypes=usedMetaCols,typeMetaData=usedMetaData,divergenceThreshold=-1)

####
# Make bar plot containing 4 major comparisons and all sufficiently succesful nmers


#############################################
# Looking at nmers in terms of DaviesBouldinIndex and cluster results, given a particular specification pair
bestNmerFeatureSets
bestNmerFeatureSets$FeatureSets
bestNmerFeatureSets$FeatureSetDbis

# Definition of axon cell type sets
spec1 <- list(ArborType="Axon",ExperCond="Control",Class2="Pyramidal cell")
spec2 <- list(ArborType="Axon",ExperCond="Control",Class1="Interneuron")
spec3 <- list(ArborType="Axon",ExperCond="Control",Class2="Uniglomerular projection neuron")
specs <- list(Pyramidal=spec1,Interneuron=spec2,UPN=spec3)

#Dendrite/Motoneuron by BrainRegion - Spinal cord vs Brainstem - TAA(.4),TA(.3),sAAA(.29),sAAT(.27),sCAA(.25),CAT(.22)
index <- grep("Dendrite/Motoneuron by BrainRegion - Spinal cord vs Brainstem",names(PairwiseSpecificationLists))
#ApicalDendrite/Rodent/Pyramidal cell by BrainRegion - Hippocampus vs Anterior olfactory nucleus - TCA(~.23), CAA(~.2), AAT(.18), sCAA(.163), TCT(.149), AT(.14), CAC(.14), ACT(.13)
index <- grep("ApicalDendrite/Rodent/Pyramidal cell by BrainRegion - Hippocampus vs Anterior olfactory nucleus",names(PairwiseSpecificationLists))
index <- grep("ApicalDendrite/Rodent/Pyramidal cell by BrainRegion - Neocortex vs Hippocampus",names(PairwiseSpecificationLists))
#Dendrite/Primate/Neocortex/Layer 2,3/Pyramidal cell by CorticalRegion - Prefrontal vs Temporal sulcus - sAAA(.182)
index <- grep("^Dendrite/Primate/Neocortex/Layer 2,3/Pyramidal cell by CorticalRegion - Prefrontal vs Temporal sulcus",names(PairwiseSpecificationLists))
#ApicalDendrite/Primate/Neocortex/Pyramidal cell by AgeClass - Adult vs Young - ACA(.171), TCA(.118), TAA(.114)
index <- grep("ApicalDendrite/Primate/Neocortex/Pyramidal cell by AgeClass - Adult vs Young",names(PairwiseSpecificationLists))
#ApicalDendrite/Neocortex/Pyramidal cell by Order - Rodent vs Elephant - TCT(.169), CCT(.094)
index <- grep("ApicalDendrite/Neocortex/Pyramidal cell by Order - Rodent vs Elephant",names(PairwiseSpecificationLists))
#Dendrite/Retina/Ganglion cell by Species - Mouse vs Salamander - sAAA(.138), CAA(.096), TT(.095), sCAA(.091), CT(.085)
index <- grep("Dendrite/Retina/Ganglion cell by Species - Mouse vs Salamander",names(PairwiseSpecificationLists))
#Axon/Rodent/Neocortex/Pyramidal cell by CorticalRegion - Somatosensory vs Visual - C(.129)
index <- grep("Axon/Rodent/Neocortex/Pyramidal cell by CorticalRegion - Somatosensory vs Visual",names(PairwiseSpecificationLists))
#ApicalDendrite/Primate/Neocortex/Prefrontal/Layer 2,3/Pyramidal cell by Species - Monkey vs Human - TCA(.121), ACA(.083)
index <- grep("ApicalDendrite/Primate/Neocortex/Prefrontal/Layer 2,3/Pyramidal cell by Species",names(PairwiseSpecificationLists))

# MinBifs22
# C .24 (FreqDiff); .245 (PercRank)
index <- grep("Axon/Rodent/Neocortex/Pyramidal cell by Species - Mouse vs Rat",names(PairwiseSpecificationLists))
# C .165 (FreqDiff); .154 (PercRank)
index <- grep("Axon/Rodent/Interneuron by Class2 - Basket cell vs Martinotti cell&Somatostatin \\(SOM\\) containing cell",names(PairwiseSpecificationLists))
# C .138 (FreqDiff); .158 (PercRank)
index <- grep("Axon/Rodent/Neocortex/Pyramidal cell by CorticalRegion - Somatosensory vs Visual",names(PairwiseSpecificationLists))
# C .05 (FreqDiff); .14 (PercRank)
index <- grep("Axon/Rodent/Young/Neocortex/Pyramidal cell by Layer - Layer 5 vs Layer 2/3",names(PairwiseSpecificationLists))
# C .03 (FreqDiff); .11 (PercRank)
index <- grep("Axon/Rodent/Neocortex/Pyramidal cell by Layer - Layer 5 vs Layer 2/3",names(PairwiseSpecificationLists))


specifications <- SpecificationLists[[comparisonName]]
specifications <- SpecificationLists[[index]]
specifications <- SelectSpecificationLists[[index]]

specifications <- list(Pyramidal=spec1,Interneuron=spec2)
specifications <- list(Pyramidal=spec1,UPN=spec3)
specifications <- list(Interneuron=spec2,UPN=spec3)
specifications <- list(Rodent=specifications$Rodent,Elephant=specifications$Elephant)
specifications <- list(Adult=specifications$Adult,Young=specifications$Young)
specifications <- list(Hippocampus=specifications$Hippocampus,AON=specifications$AnteriorOlfactoryNucleus)

minBifs <- 0
minBifs <- 22
specifications <- PairwiseSpecificationLists[[index]]
specTitle <- names(PairwiseSpecificationLists)[index]
groupedSeqs <- getSpecificationGroups(specifications,presentMetaData,minBifs=minBifs)
specSeqs <- c()
groupings <- c()
for (i in 1:length(groupedSeqs)){
  specSeqs <- append(specSeqs,groupedSeqs[[i]])
  groupings <- append(groupings,rep(i,length(groupedSeqs[[i]])))
}
partitions <- groupings
nmers <- nmersXformed[specSeqs,]
morphometrics <- morphometricsXformed[specSeqs,]
rawMetrics <- Morphometrics[specSeqs,]
rawPRs <- PRVect[specSeqs,]
nmerFDs <- NFVect[specSeqs,]
sampleSize <- min(100,floor(0.8*min(sapply(groupedSeqs,length))))

archiveMetrics <- archiveMetricsXformed[specSeqs,]

nmerClustRIs <- list()
nmerClustRIs[[3]] <- basicClusterResults(nmers[,3,drop=F],partitions=partitions)
nmerClustRIs[[i]] <- basicClusterResults(nmers[,i,drop=F],partitions=partitions)

summary(basicClusterResults(nmers[,2,drop=F],partitions=groupings,sampleSize=sampleSize,numClusterRuns=30,sampleProportionally=F))
summary(basicClusterResults(data.frame(TCT=nmers$TCT,morphometrics$BranchOrder),partitions=groupings,sampleSize=sampleSize,numClusterRuns=30,sampleProportionally=F))
rivals <- basicClusterResults(as.data.frame(nmers$TCA),partitions=groupings,sampleSize=sampleSize,numClusterRuns=40,sampleProportionally=F)
summary(rivals)
rivals <- basicClusterResults(as.data.frame(nmers$TAT),partitions=groupings,sampleSize=sampleSize,numClusterRuns=20,sampleProportionally=F)
rivals <- basicClusterResults(as.data.frame(morphometrics$Branches),partitions=groupings,sampleSize=sampleSize,numClusterRuns=20,sampleProportionally=F)
rivals <- basicClusterResults(data.frame(nmers$TAT,morphometrics$Branches),partitions=groupings,sampleSize=sampleSize,numClusterRuns=20,sampleProportionally=F)

rivals <- basicClusterResults(as.data.frame(nmerFDs$C),partitions=groupings,sampleSize=sampleSize,numClusterRuns=30,sampleProportionally=F)
summary(rivals)
rivals <- basicClusterResults(as.data.frame(nmers$C),partitions=groupings,sampleSize=sampleSize,numClusterRuns=30,sampleProportionally=F)

summary(basicClusterResults(as.data.frame(nmers$sAAA),partitions=groupings,sampleSize=sampleSize,numClusterRuns=30,sampleProportionally=F))
summary(basicClusterResults(data.frame(sAAA=nmers$sAAA,PKClassic=morphometrics$PKClassic),partitions=groupings,sampleSize=sampleSize,numClusterRuns=30,sampleProportionally=F))
plot(morphometrics$Branches,nmers$C,col=partitions)
plot(morphometrics$Branches,nmerFDs$sAAA,col=partitions)
plot(morphometrics$Branches,nmerFDs$C,col=partitions)
plot(nmers$C,nmerFDs$C,col=partitions)
plot(morphometrics$Branches[rawMetrics$Branches>21],nmers$C[rawMetrics$Branches>21],col=partitions)
plot(morphometrics$PKClassic,nmers$TCA,col=partitions)
plot(morphometrics$Branches,nmers$TCA,col=partitions)
plot(rawMetrics$Branches,nmerFDs$ACA,col=partitions)

plot(morphometrics$Depth,nmerFDs$C,col=partitions)
plot(morphometrics$PartitionAsymmetry,nmerFDs$C,col=partitions)
plot(morphometrics$Branches,nmerFDs$ACA,col=partitions)

plot(morphometrics$Branches,nmers$C,col=partitions,cex=.6+.4*(rawMetrics$Branches>=22),pch=tmpPart)
plot(morphometrics$Branches,nmerFDs$C,col=partitions,cex=.6+.4*(rawMetrics$Branches>=22),pch=tmpPart)
plot(morphometrics$BranchOrder,nmerFDs$C,col=partitions,cex=.6+.4*(rawMetrics$Branches>=22))
rivalsC <- basicClusterResults(as.data.frame(nmers$C),partitions=groupings,sampleSize=sampleSize,numClusterRuns=30,sampleProportionally=F)
rivalsC2 <- basicClusterResults(as.data.frame(nmerFDs$C),partitions=groupings,sampleSize=sampleSize,numClusterRuns=30,sampleProportionally=F)
summary(rivalsC)
rivalsBO <- basicClusterResults(as.data.frame(morphometrics$BranchOrder),partitions=groupings,sampleSize=sampleSize,numClusterRuns=30,sampleProportionally=F)
summary(rivalsBO)
rivalsC_BO <- basicClusterResults(data.frame(nmerFDs$C,morphometrics$BranchOrder),partitions=groupings,sampleSize=sampleSize,numClusterRuns=30,sampleProportionally=F)
summary(rivalsC_BO)
rivalsCMinBifs <- basicClusterResults(as.data.frame(nmers$C[rawMetrics$Branches>=22]),partitions=groupings[rawMetrics$Branches>=22],sampleSize=sampleSize,numClusterRuns=30,sampleProportionally=F)

rivalsFrag <- basicClusterResults(as.data.frame(archiveMetrics$Fragmentation),partitions=groupings,sampleSize=sampleSize,numClusterRuns=30,sampleProportionally=F)
rivalsFragMinBifs <- basicClusterResults(as.data.frame(archiveMetrics$Fragmentation[rawMetrics$Branches>=22]),partitions=groupings[rawMetrics$Branches>=22],sampleSize=sampleSize,numClusterRuns=30,sampleProportionally=F)

plot((rawMetrics$Branches-1)/2,nmers$C,col=partitions,cex=.7,xlim=c(0,200),xlab="Number of Bifurcations",ylab="C frequency difference (neurite-baseline avg)",main=sub(" by BrainRegion - ","\n",specTitle))

plot((rawMetrics$Branches-1)/2,nmers$TCA,col=partitions,cex=.7,xlim=c(0,200),xlab="Number of Bifurcations",main=sub(" by BrainRegion - ","\n",specTitle))
plot((rawMetrics$Branches-1)/2,nmers$TCA,col=partitions,cex=.7,xlim=c(0,200),xlab="Number of Bifurcations",main=sub(" cell by BrainRegion - ","\n",specTitle))

plot((rawMetrics$Branches-1)/2,nmerFDs$C,col=partitions,cex=.7,xlim=c(0,50),xlab="Number of Bifurcations",ylab="C frequency difference (neurite-baseline avg)",main=sub(" cell by Species - ","\n",specTitle))
legend(150,-.05,c("Mouse","Rat"),cex=.7,pch=1,col=1:2)

plot((rawMetrics$Branches-1)/2,nmers$ACA,col=partitions,cex=.7,xlim=c(0,50),xlab="Number of Bifurcations",ylab="Normalized ACA Percentile Ranks",main=sub(" cell by AgeClass - ","\n",specTitle))
legend(40,-.5,c("Adult","Young"),cex=.7,pch=1,col=1:2)

plot((rawMetrics$Branches-1)/2,nmers$TAA,col=partitions,cex=.7,xlab="Number of Bifurcations",ylab="Normalized TAA Percentile Ranks",main=sub(" - ","\n",specTitle),xlim=c(0,200))
legend(120,-.2,c("Spinal cord","Brainstem"),cex=.8,pch=1,col=1:2)

plot((rawMetrics$Branches-1)/2,nmers$TCA,col=partitions,cex=.7,xlab="Number of Bifurcations",ylab="Normalized TCA Percentile Ranks",main=sub(" cell by BrainRegion - ","\n",specTitle),xlim=c(0,100))
legend(0,-1.2,c("Hippocampus","Ant. olf. nuc."),cex=.8,pch=1,col=1:2)

# See clustering scores of Axon/Rodent/Neocortex/Pyramidal cell by CorticalRegion - Somatosensory vs Visual
# Look for archive effect: Run just Yuste across Somatosensory and Visual
# However this could be showing Species effect rather than Archive effect
yusteSpec <- specifications
yusteSpec[[1]] <- append(yusteSpec[[1]],list(Archive="Yuste"))
yusteSpec[[2]] <- append(yusteSpec[[2]],list(Archive="Yuste"))
yusteGroupedSeqs <- getSpecificationGroups(yusteSpec,presentMetaData)
yusteSeqs <- c()
yusteGroupings <- c()
for (i in 1:length(groupedSeqs)){
  yusteSeqs <- append(yusteSeqs,yusteGroupedSeqs[[i]])
  yusteGroupings <- append(yusteGroupings,rep(i,length(yusteGroupedSeqs[[i]])))
}
sampleSize <- min(100,floor(0.8*min(sapply(yusteGroupedSeqs,length))))
rivals <- basicClusterResults(as.data.frame(nmers[yusteSeqs,"C"]),partitions=yusteGroupings,sampleSize=sampleSize,numClusterRuns=30,sampleProportionally=F)



tmpClust <- hclust(dist(nmers$C[rawMetrics$Branches>=22]))
tmpPart <- cutree(tmpClust,2)
AdjustedRandIndex(tmpPart,groupings[rawMetrics$Branches>=22])

rivalsTCA <- basicClusterResults(as.data.frame(nmers$TCA),partitions=groupings,sampleSize=sampleSize,numClusterRuns=30,sampleProportionally=F)
rivalsPKC <- basicClusterResults(as.data.frame(morphometrics$PKClassic),partitions=groupings,sampleSize=sampleSize,numClusterRuns=30,sampleProportionally=F)
rivalsLength <- basicClusterResults(as.data.frame(morphometrics$Length),partitions=groupings,sampleSize=sampleSize,numClusterRuns=30,sampleProportionally=F)
rivalsPKC_TCA <- basicClusterResults(data.frame(nmers$TCA,morphometrics$PKClassic),partitions=groupings,sampleSize=sampleSize,numClusterRuns=30,sampleProportionally=F)
rivalsLength_TCA <- basicClusterResults(data.frame(nmers$TCA,morphometrics$Length),partitions=groupings,sampleSize=sampleSize,numClusterRuns=30,sampleProportionally=F)


basicClusterResults(as.data.frame(nmers[,c(3,8)]),partitions=partitions,sampleSize=sampleSize,numClusterRuns=30)

# Proportional, larger subsample
rivals <- basicClusterResults(as.data.frame(nmers$sAAA),partitions=groupings,sampleSize=60,numClusterRuns=20,sampleProportionally=T)
summary(basicClusterResults(as.data.frame(nmers$sAAA),partitions=groupings,sampleSize=120,numClusterRuns=20,sampleProportionally=T))

mean(basicClusterResults(as.data.frame(morphometrics[,"Branches"]),partitions=groupings,sampleSize=60,numClusterRuns=30))


DaviesBouldinIndex(partition=neuriteClasses,featureVector=nmers[,c(8)])
DaviesBouldinIndex(partition=neuriteClasses,featureVector=nmers[,c(18)])
DaviesBouldinIndex(partition=neuriteClasses,featureVector=nmers[,c(8,18)])
DaviesBouldinIndex(partition=neuriteClasses,featureVector=nmers[,c(8,5)])

DaviesBouldinIndex(partition=neuriteClasses,featureVector=morphometrics[,c("Height","Length")])
DaviesBouldinIndex(partition=neuriteClasses,featureVector=morphometrics[,c("Height")])
DaviesBouldinIndex(partition=neuriteClasses,featureVector=morphometrics[,c("Length")])
summary(basicClusterResults(as.data.frame(morphometrics[,c("Height")]),partitions=partitions,sampleSize=60,numClusterRuns=30))
summary(basicClusterResults(as.data.frame(morphometrics[,c("Branches")]),partitions=partitions,sampleSize=60,numClusterRuns=30))
summary(basicClusterResults(as.data.frame(morphometrics[,c("Length")]),partitions=partitions,sampleSize=60,numClusterRuns=30))
summary(basicClusterResults(as.data.frame(morphometrics[,c("Height","Length")]),partitions=partitions,sampleSize=60,numClusterRuns=30))

summary(basicClusterResults(as.data.frame(morphometrics[,c("Branches")]),partitions=partitions,sampleSize=60,numClusterRuns=30))
summary(basicClusterResults(as.data.frame(morphometrics[,c("Branches")]),partitions=partitions,sampleSize=60,numClusterRuns=30,method="kmeans"))

summary(basicClusterResults(as.data.frame(morphometrics[,c("Length")]),partitions=partitions,sampleSize=60,numClusterRuns=30,method="kmeans"))
summary(basicClusterResults(as.data.frame(morphometrics[,c("Height","Length")]),partitions=partitions,sampleSize=60,numClusterRuns=30,method="kmeans"))

DaviesBouldinIndex(partition=neuriteClasses,featureVector=cbind(nmers$TCT,morphometrics$Length))
DaviesBouldinIndex(partition=neuriteClasses,featureVector=as.data.frame(nmers$TCT))

plot(morphometrics$Height,morphometrics$Length,col=partitions)
plot(morphometrics$Branches,morphometrics$Length,col=partitions)
plot(morphometrics$Branches,nmers$TCA,col=partitions)
plot(rawMetrics$Branches,nmers$TCA,col=partitions)
plot(1:nrow(nmers),nmers$TCT,col=partitions)
plot(1:nrow(nmers),nmers$sAAA,col=partitions)
plot(1:nrow(nmers),nmers$sAAA,col=partitions,pch=tmpPart)

plot(1:nrow(nmers),nmers$TCT,col=groupings,pch=21,bg=groupings*(rownames(nmers)%in%rownames(nmers)[subsample]))
plot(1:nrow(nmers),nmers$CCT,col=groupings)

plot(nmers$sCAA,morphometrics$Length,col=partitions)
plot(1:nrow(nmers),nmers$sAAA,col=partitions)
plot(nmers[,c(8)],morphometrics$Length,col=partitions)
plot(nmers[,c(8)],nmers[,c(37)],col=partitions)

plot(1:nrow(morphometrics),morphometrics$Branches,col=groupings)
plot(morphometrics$Branches,nmers$TCT,col=groupings)
plot(rawMetrics$Branches,nmers$sCAA,col=groupings)
plot(1:nrow(morphometrics),nmers$sCAA,col=groupings)


protocols <- presentMetaData[specSeqs,"Protocol"]
protocolsColors <- sapply(protocols,grep,unique(protocols))
class3s <- presentMetaData[specSeqs,"Class3"]
class3sColors <- sapply(class3s,grep,unique(class3s))

genders <- presentMetaData[specSeqs,"Gender"]
genders <- as.character(sub("M/F","M_F",genders))
gendersColors <- sapply(paste("^",genders,"$",sep=""),grep,unique(genders))
layers <- presentMetaData[specSeqs,"Layer"]
layersColors <- sapply(layers,grep,unique(layers))
regions <- presentMetaData[specSeqs,"CorticalRegion"]
regionsColors <- sapply(regions,grep,unique(regions))
ageClasses <- presentMetaData[specSeqs,"AgeClass"]
ageClassesColors <- sapply(ageClasses,grep,unique(ageClasses))
species <- presentMetaData[specSeqs,"Species"]
speciesColors <- sapply(species,grep,unique(species))
plot(1:nrow(morphometrics),morphometrics$FractalDimension,col=speciesColors)
archives <- presentMetaData[specSeqs,"Archive"]
archivesColors <- sapply(archives,grep,unique(archives))
plot(1:nrow(morphometrics),morphometrics$FractalDimension,col=archivesColors)
plot(1:nrow(morphometrics),Morphometrics[specSeqs,"FractalDimension"],col=archivesColors)
cellTypes <- presentMetaData[specSeqs,"Class2"]
cellTypes <- sub("\\)","",sub("\\(","",cellTypes))
cellTypesColors <- sapply(cellTypes,grep,unique(cellTypes))
plot(1:nrow(morphometrics),morphometrics$FractalDimension,col=cellTypesColors)
plot(1:nrow(morphometrics),Morphometrics[specSeqs,"FractalDimension"],col=cellTypesColors)

arborTypes <- presentMetaData[rownames(presentMetaData),"ArborType"]
typeCols <- sapply(paste("^",arborTypes,sep=""),grep,unique(arborTypes))
plot(Morphometrics$Contraction,Morphometrics$FractalDimension,col=typeCols)

which(!sapply(nmerClustRIs,is.null))
sapply(nmerClustRIs[!sapply(nmerClustRIs,is.null)],mean)

nmerPCA <- prcomp(nmers)

#############################################
# Worst performers; to be excluded from future runs

# Worst with no #branches cutoff, but not when cutoff is 22
"Dendrite/Rodent/Neocortex by Class2 - Basket cell vs Martinotti cell&Somatostatin (SOM) containing cell",
# Borderline when cutoff of 22
"Dendrite/Retina/Ganglion cell by Species - Salamander vs Goldfish",
"ApicalDendrite/Rodent/Neocortex/Pyramidal cell by Layer - Layer 5 vs Layer 2/3",
"ApicalDendrite/Rodent/Neocortex/Pyramidal cell by CorticalRegion - Somatosensory vs Occipital",
"ApicalDendrite/Neocortex/Pyramidal cell by Order - Rodent vs Primate",


worstPerformers <- c("Dendrite/Rodent/Neocortex/Layer 2,3/Pyramidal cell by CorticalRegion - Frontal vs Visual",
"Dendrite/Rodent/Neocortex by Class2 - Pyramidal cell vs Basket cell",
"Dendrite/Neocortex/Layer 2,3/Pyramidal cell by Species - Human vs Rat",
"Dendrite/Neocortex/Layer 2,3/Pyramidal cell by Species - Human vs Elephant",
"Axon/Rodent/Interneuron by Class2 - PV containing cell vs Martinotti cell&Somatostatin (SOM) containing cell",
"Dendrite/Neocortex/Layer 2,3/Pyramidal cell by Species - Rat vs Elephant",
"Axon/Rodent/Interneuron by Class2 - Basket cell vs Parvalbumin (PV) containing cell",
"Dendrite/Rodent/Neocortex/Somatosensory/Pyramidal cell by Layer - Layer 2/3 vs Layer 4",
"Dendrite/Neocortex/Pyramidal cell by Species - Monkey vs Elephant",
"Dendrite/Rodent/Neocortex by Class2 - Pyramidal cell vs Parvalbumin (PV) containing cell",
"Dendrite/Rodent/Neocortex/Somatosensory/Pyramidal cell by Layer - Layer 4 vs Layer 5",
"ApicalDendrite/Rodent/Neocortex/Pyramidal cell by CorticalRegion - Frontal vs Occipital",
"ApicalDendrite/Rodent/Neocortex/Pyramidal cell by AgeClass - Young vs Adult",
"Dendrite/Neocortex/Layer 2,3/Pyramidal cell by Species - Monkey vs Rat",
"Dendrite/Rodent/Neocortex by Class2 - PV containing cell vs Martinotti cell&Somatostatin (SOM) containing cell",
"Dendrite/Neocortex/Layer 2,3/Pyramidal cell by Species - Human vs Mouse",
"Dendrite/Neocortex/Layer 2,3/Pyramidal cell by Species - Monkey vs Mouse",
"Dendrite/Rodent/Neocortex/Layer 2,3/Pyramidal cell by CorticalRegion - Somatosensory vs Visual",
"Dendrite/Neocortex/Pyramidal cell by Species - Monkey vs Mouse",
"Dendrite/Neocortex/Pyramidal cell by Species - Monkey vs Rat",
"ApicalDendrite/Rodent/Neocortex/Pyramidal cell by CorticalRegion - Somatosensory vs Frontal",
"Dendrite/Neocortex/Layer 2,3/Pyramidal cell by Order - Primate vs Rodent",
"Dendrite/Neocortex/Layer 2,3/Pyramidal cell by Species - Mouse vs Rat",
"Dendrite/Rodent/Young/Neocortex/Somatosensory/Layer 4/Pyramidal cell by Species - Mouse vs Rat",
"Dendrite/Neocortex/Pyramidal cell by Species - Rat vs Elephant",
"Dendrite/Primate/Neocortex/Layer 2,3/Pyramidal cell by CorticalRegion - Anterior long insular gyrus vs Posterior short insular gyrus",
"Axon/Rodent/Young/Neocortex/Pyramidal cell by Layer - Layer 4 vs Layer 2/3",
"Axon/Rodent/Neocortex/Pyramidal cell by Layer - Layer 4 vs Layer 2/3",
"Dendrite/Neocortex/Pyramidal cell by Species - Mouse vs Elephant",
"Dendrite/Primate/Neocortex/Layer 2,3/Pyramidal cell by CorticalRegion - Anterior long insular gyrus vs Middle short insular gyrus",
"Dendrite/Primate/Neocortex/Layer 2,3/Pyramidal cell by CorticalRegion - Posterior short insular gyrus vs Middle short insular gyrus",
"Dendrite/Neocortex/Pyramidal cell by Species - Mouse vs Rat",
"Dendrite/Human/Neocortex/Pyramidal cell Gender - M vs F")

worstPerformersMinBif22 <- c("ApicalDendrite/Rodent/Neocortex/Pyramidal cell by Layer - Layer 5 vs Layer 2/3",
"Dendrite/Neocortex/Pyramidal cell by Species - Monkey vs Rat",
"Axon/Rodent/Interneuron by Class2 - Basket cell vs Parvalbumin (PV) containing cell",
"Axon/Rodent/Neocortex/Pyramidal cell by Layer - Layer 4 vs Layer 5",
"Dendrite/Neocortex/Pyramidal cell by Species - Human vs Rat",
"Dendrite/Rodent/Neocortex by Class2 - Pyramidal cell vs Basket cell",
"Axon/Rodent/Young/Neocortex/Pyramidal cell by Layer - Layer 4 vs Layer 5",
"Dendrite/Rodent/Neocortex by Class2 - Pyramidal cell vs Parvalbumin (PV) containing cell",
"Dendrite/Neocortex/Pyramidal cell by Species - Human vs Mouse",
"Dendrite/Neocortex/Layer 2,3/Pyramidal cell by Species - Human vs Monkey",
"Axon/Rodent/Young/Neocortex/Somatosensory/Pyramidal cell by Layer - Layer 4 vs Layer 5",
"Axon/Rodent/Young/Neocortex/Pyramidal cell by Layer - Layer 4 vs Layer 2/3",
"Axon/Rodent/Neocortex/Pyramidal cell by Layer - Layer 4 vs Layer 2/3",
"Dendrite/Neocortex/Layer 2,3/Pyramidal cell by Species - Human vs Mouse",
"Dendrite/Human/Neocortex/Layer 2,3/Pyramidal cell/In vitro by CorticalLobe - Insula vs Frontal",
"Dendrite/Neocortex/Pyramidal cell by Order - Rodent vs Elephant",
"ApicalDendrite/Rodent/Young/Pyramidal cell by BrainRegion - Neocortex vs Hippocampus",
"Dendrite/Rodent/Neocortex by Class2 - Parvalbumin (PV) containing cell vs Martinotti cell&Somatostatin (SOM) containing cell",
"Dendrite/Neocortex/Pyramidal cell by Order - Primate vs Rodent",
"ApicalDendrite/Rodent/Pyramidal cell by BrainRegion - Neocortex vs Hippocampus",
"Dendrite/Neocortex/Pyramidal cell by Species - Rat vs Elephant",
"Dendrite/Neocortex/Layer 2,3/Pyramidal cell by Species - Monkey vs Mouse",
"Dendrite/Neocortex/Layer 2,3/Pyramidal cell by Order - Primate vs Rodent",
"Dendrite/Neocortex/Pyramidal cell by Species - Monkey vs Mouse",
"Dendrite/Neocortex/Pyramidal cell by Species - Mouse vs Rat",
"Dendrite/Human/Neocortex/Pyramidal cell Gender - M vs F")

#############################################
# Explore relationships between variables

#index <- grep("Dendrite/Motoneuron by BrainRegion",names(SpecificationLists))
specifications <- SpecificationLists[[index]]
specTitle <- names(SpecificationLists)[index]
numGroups <- length(specifications)
groupSeqs <- getSpecificationGroups(specifications,usedMetaData)
allSeqs <- c()
seqLabels <- c()
for (i in 1:length(groupSeqs)){
  allSeqs <- append(allSeqs,groupSeqs[[i]])
  seqLabels <- append(seqLabels,rep(i,length(groupSeqs[[i]])))
}
x11()
plot(PRVect[allSeqs,1:10],col=seqLabels)



###############################################################
# Testing clustering results given various models
#

# 2 relatively highly correlated features
# 2 classes
# No difference between classes on feature 1 (which has greater variance)
# Small mean difference between classes on feature 2
tmpa <- rnorm(100,30,5)
tmpb <- tmpa+rnorm(100,0,5)
tmpc <- rnorm(100,30,5)
tmpd <- tmpc+rnorm(100,8,5)
tmpac <- c(tmpa,tmpc)
tmpbd <- c(tmpb,tmpd)
tmpdf <- data.frame(AC=tmpac,BD=tmpbd)
tmpClasses <- c(rep(1,100),rep(2,100))

# Cluster on all the data - unscaled
clusterAndPlot(tmpdf,tmpClasses,tmpdf)

# Cluster on each feature
clusterAndPlot(tmpdf$AC,tmpClasses,tmpdf)
clusterAndPlot(tmpdf$BD,tmpClasses,tmpdf)

tmpPCs <- prcomp(tmpdf,scale.=T)
clusterAndPlot(tmpPCs$x,tmpClasses,tmpdf)
clusterAndPlot(tmpPCs$x[,1],tmpClasses,tmpdf)
clusterAndPlot(tmpPCs$x[,2],tmpClasses,tmpPCs$x)

clusterAndPlot <- function(mydf,classes,originalDf){
  tmpdist <- dist(mydf)
  tmpClust <- hclust(tmpdist,"ward")
  tmpPart <- cutree(tmpClust,2)
  print(AdjustedRandIndex(tmpClasses,tmpPart))
  plot(originalDf,col=classes,pch=tmpPart)
}

test1A<-rnorm(100,10,4)
test2A<-rnorm(100,13,4)
test1B<-rnorm(100,10,4)
test2B<-rnorm(100,13,4)
testGroups <- c(rep(1,100),rep(2,100))
testData <- data.frame(FA=c(test1A,test2A),FB=c(test1B,test2B))
testDist <- dist(testData)
testDist <- dist(testData[,1])
testDist <- dist(testData[,2])
testClust <- hclust(testDist,"ward")
testPart <- cutree(testClust,2)
AdjustedRandIndex(testPart,testGroups)
plot(testData,col=testGroups,pch=testPart)

testPC <- prcomp(testData,scale=T)
testDist <- dist(testPC$x[,1])
testDist <- dist(testPC$x[,2])
testClust <- hclust(testDist,"ward")
testPart <- cutree(testClust,2)
AdjustedRandIndex(testPart,testGroups)
plot(testData,col=testGroups,pch=testPart)

