#library("nortest")
#source("C:/Documents and Settings/gillette/My Documents/My Dropbox/Motif Analysis/Motif Analysis R Files/Utils.r")

defaultDirectory <- ''
defaultOrder <- "LTS"
defaultMethod <- 8
defaultThreshold <- 0
defaultThreshType <- "Whole"
nullHSets <- 20

#actualNumKmers <- c(1,9,25,68,183)

#metaData <- read.csv("C:/Documents and Settings/gillette/My Documents/research/MotifAnalysis/arborMetaData-Map2.csv")

getOutliers <- function(kPlus,sets=20,surrogateSet=-1,order=defaultOrder,sourceTraversal=order,method=defaultMethod,threshold=defaultThreshold,thresholdTypeString=defaultThreshType,directory=defaultDirectory,useRandomSet=-1,useRandomK=0){
  outliers <- read.csv(getOutliersFilename(kPlus,sets,surrogateSet,order,sourceTraversal,method,threshold,thresholdTypeString,directory,useRandomSet,useRandomK),stringsAsFactors=F)
  numKmers <- length(getKmersForOutlierAnalysis(kPlus,outliers))
  outliers$OutlierFraction <- apply(outliers[3:(2+actualNumKmers[kPlus])],1,mean)
#  outliers$OutlierFraction <- apply(outliers[numKmers],1,mean)
  outliers
}

getOutliersFilename <- function(kPlus,sets=20,surrogateSet=-1,order=defaultOrder,sourceTraversal=order,method=defaultMethod,threshold=defaultThreshold,thresholdTypeString=defaultThreshType,directory=defaultDirectory,useRandomSet=-1,useRandomK=0){
  filename <- paste(directory,"MotifOutliers-",sets,"Sets",sep="")
  if (surrogateSet > -1){
    filename <- paste(filename,"-",surrogateSet,sep="")
  }
  if (useRandomSet > -1){
    filename <- paste(filename,"-RandSet",useRandomSet,"_",useRandomK,sep="")
  }
  filename <- paste(filename,"-Size",order,"-B",method,"-T",threshold,"-",thresholdTypeString,"_K",(kPlus-1),"-",kPlus,".csv",sep="")
  filename
}

getNullHData <- function(randSet,kPlus,sets=20,order=defaultOrder,method=defaultMethod,threshold=defaultThreshold,directory=defaultDirectory){
  if (!exists("nullHData",envir=globalenv())){
    outlierNullHData <- getOutliers(kPlus,sets=20,surrogateSet=1,order=order,method=method,threshold=threshold,directory=directory,useRandomSet=randSet,useRandomK=(kPlus-1))
  }
  else{
    nullHData <- get("nullHData",envir=globalenv())
    if (length(nullHData) < kPlus || is.null(nullHData[[kPlus]])){
      nullHData[[kPlus]] <- list()
    }
    if (length(nullHData[[kPlus]]) < randSet || is.null(nullHData[[kPlus]][[randSet]])){
      outlierNullHData <- getOutliers(kPlus,sets=20,surrogateSet=1,order="LTS",method=8,threshold="0.0",directory=directory,useRandomSet=randSet,useRandomK=(kPlus-1))
      kmers <- getKmersForOutlierAnalysis(kPlus,outlierNullHData)
      outlierNullHData <- outlierNullHData[,c(kmers,"OutlierFraction")]
      nullHData[[kPlus]][[randSet]] <- outlierNullHData
    }
    outlierNullHData <- nullHData[[kPlus]][[randSet]]
    assign("nullHData", nullHData, envir=globalenv())
  }
  outlierNullHData
}

outlierBinomialFit <- function(outliers,k=2){
  kmers <- generateKmers(k)
  numKmers <- length(kmers)
  ecdfData <- ecdf(outliers$OutlierFraction*numKmers)
  x <- 0:numKmers
  y <- ecdfData(x)
  fit <- nls(y ~ pbinom(x,numKmers,p),start=c(p=.05))
  fit
}

findSignificantKmers <- function(realOutliers,randomOutliers,k,alpha=0.05,arborType=NA,cellType=NA,brainRegion=NA,minLength=NaN,maxLength=NaN,histogram=F){
  kmers <- generateKmers(k)
  numSignificant <- 0
  totalPossible <- 0
  for (kmer in kmers){
    if (kmer %in% colnames(mydata)){
      totalPossible <- totalPossible + 1
      subData = getSubset(mydata,arborType,cellType,brainRegion,minLength,maxLength)
      if (!is.null(varData)){
        sequences = as.character(varData[varData[,kmer]!=0,"Sequence"])
        subData = subset(subData,Sequence%in%sequences)
      }
      # Using the Anderson-Darling test for normality on a normalized version of the data
      normedData <- qnorm(subData[,kmer])
      testStat <- ad.test(normedData)
      if (testStat$p.value <= alpha){
        numSignificant <- numSignificant + 1
        print(paste(kmer,testStat$p.value))
        if (histogram){
          percentileRanksHistogram(subData,kmer,arborType=arborType,cellType=cellType,brainRegion=brainRegion,minLength=minLength,maxLength=maxLength)
        }
      }
    }
  }
  print(paste(numSignificant," significant ",k,"-mers out of ",totalPossible,sep=""))
}

calculateConditionalProbabilities <- function(outlierData,k){
  kmers <- generateKmers(k)
  numKmers <- actualNumKmers[k]
  kmers <- kmers[kmers%in%colnames(outlierData)]
  probMatrix <- matrix(0,numKmers,numKmers)
  colnames(probMatrix) <- kmers
  rownames(probMatrix) <- kmers
  for (i in kmers){
    for (j in kmers){
      probMatrix[i,j] <- mean(outlierData[,i]&outlierData[,j])/mean(outlierData[,j])
    }
  }
  probMatrix
}

getOutliersList <- function(k,sets,random=T,randomSet=-1,order="LTS",directory=defaultDirectory){
  outliersList <- list()
  for (i in 1:sets){
    useRandomSet <- -1
    if (random){
      if (randomSet > -1){
        useRandomSet <- randomSet
      }
      else{
        useRandomSet <- i
      }
    }
    outliersList[[i]] <- getOutliers(k,sets=20,surrogateSet=i,order=order,method=8,threshold="0.0",useRandomSet=useRandomSet,useRandomK=k-1,directory=directory)
  }
  outliersList
}

baselineKmerSignificanceDistribution <- function(k,outliersList,title=NA){
  kmers <- generateKmers(k)
  numKmers <- actualNumKmers[k]
  kmers <- kmers[kmers%in%colnames(outliersList[[1]])]
  pVals <- c()
  wVals <- c()
  for (i in 1:(length(outliersList)-1)){
    for (j in (i+1):length(outliersList)){
      for (kmer in kmers){
        teststat <- wilcox.test(outliersList[[i]][,kmer],outliersList[[j]][,kmer])
        pVals <- c(pVals,teststat$p.value)
        wVals <- c(wVals,teststat$statistic)
      }
    }
  }
  wTitle = "W Statistics"
  pTitle = "P-Values"
  if (!is.na(title)){
    wTitle <- paste(wTitle,"for",title)
    pTitle <- paste(pTitle,"for",title)
  }
  x11()
  hist(wVals,main=wTitle)
  par(ask=T)
  hist(pVals,breaks=20,main=pTitle)
  par(ask=F)
}

outlierFractionSignificanceDistribution <- function(outliersList,title=NA,test=c("wilcoxin","binomial","poisson")){
  pVals <- c()
  for (i in 1:(length(outliersList)-1)){
    for (j in (i+1):length(outliersList)){
      if (test == "wilcoxin"){
        teststat <- wilcox.test(outliersList[[i]]$OutlierFraction,outliersList[[j]]$OutlierFraction)
      }
#      else if (test == "binomial"){
#        teststat <- binom.test(sum(outliersList[[i]]$OutlierFraction),length(outliersList[[i]]$OutlierFraction),mean(outliersList[[j]][,kmer]))
#      }
      else{
        teststat <- poisson.test(c(sum(outliersList[[i]][,kmer]),sum(outliersList[[j]][,kmer])),length(outliersList[[i]][,kmer]))
      }
      pVals <- c(pVals,teststat$p.value)
    }
  }
  x11()
  if (is.na(title)){
    hist(pVals,breaks=20)
  }
  else{
    hist(pVals,main=title,breaks=20)
  }
  pVals
}

getRatioMeans <- function(outliersList){
  ratioMeans <- c()
  for (i in 1:length(outliersList)){
    ratioMeans <- c(ratioMeans,mean(outliersList[[i]]$OutlierFraction))
  }
  ratioMeans
}

getRatioSDs <- function(outliersList){
  ratioSDs <- c()
  for (i in 1:length(outliersList)){
    ratioSDs <- c(ratioSDs,sd(outliersList[[i]]$OutlierFraction))
  }
  ratioSDs
}

plotBinomialDataAndFits <- function(real,random,k){
  kmers <- generateKmers(k)
  numKmers <- actualNumKmers[k]

  realFit<-outlierBinomialFit(real,k)
  randFit<-outlierBinomialFit(random,k)

  ecdfData <- ecdf(real$OutlierFraction*numKmers)
  ecdfRandData <- ecdf(random$OutlierFraction*numKmers)
  x11()
  x <- 0:numKmers
  plot(x,ecdfData(x),main="Cumulative Distribution",xlab="Number of kmer outliers in a sequence",ylab="Cumulative Frequency")
  lines(x,pbinom(x,numKmers,coef(realFit)["p"]),col=1)
  points(x,ecdfRandData(x),col=2,pch=2)
  lines(x,pbinom(x,numKmers,coef(randFit)["p"]),col=2)
  leg.txt <- c("Real CDF and Fit","Random CDF and Fit")
  legend(numKmers*0.6,0.9,leg.txt,bg="white",col=c(1,2),pch=c(1,2),lty=c(1,1))

  par(ask=T)
  plot(x,ecdfRandData(x)-c(0,ecdfRandData(0:(numKmers-1))),col=2,xlab="Number of kmer outliers in a sequence",ylab="Frequency")
  lines(x,dbinom(x,numKmers,coef(randFit)["p"]),col=2)
  points(x,ecdfData(x)-c(0,ecdfData(0:(numKmers-1))),col=1,pch=2)
  lines(x,dbinom(x,numKmers,coef(realFit)["p"]),col=1)
  leg.txt <- c("Real PDF and Fit","Random PDF and Fit")
  legend(numKmers*0.6,0.9,leg.txt,bg="white",col=c(1,2),pch=c(1,2),lty=c(1,1))

  par(ask=F)
}

# Lower triangle is coocurrance, upper triangle is deviation from expectation
calculateOutlierCoocurrence <- function(motifOutliers,k,alpha=0.05,chisq=F,deviation="Relative"){
  numKmers <- actualNumKmers[k]
  lowerBound = alpha/2
  upperBound = 1-lowerBound
  kmers <- generateKmers(k)
  kmers <- kmers[kmers%in%colnames(motifOutliers)]
  outlierTable <- motifOutliers[,kmers]
  coocurrence <- matrix(0,nrow=numKmers,ncol=numKmers)
  colnames(coocurrence) <- kmers
  rownames(coocurrence) <- kmers
  numSequences <- dim(prRanks)[1]
  for (i in 1:(numKmers-1)){
    for (j in (i+1):numKmers){
      coocurrence[j,i] <- sum(outlierTable[,i]&outlierTable[,j])/numSequences
      
      expect <- sum(outlierTable[,i])*sum(outlierTable[,j])/(numSequences^2)
      if (grepl("rel",deviation,ignore.case=T)){
        coocurrence[i,j] <- coocurrence[j,i] / expect
      }
      else if (grepl("abs",deviation,ignore.case=T)){
        coocurrence[i,j] <- coocurrence[j,i] - expect
      }
      else if (grepl("cor",deviation,ignore.case=T)){
        coocurrence[i,j] <- cor(outlierTable[,i],outlierTable[,j])
      }
      # Proportional
      else {
        coocurrence[i,j] <- coocurrence[j,i]/(1 - expect)
      }

      if (chisq){
        coocurrence[j,i] <- log(chisq.test(outlierTable[,i],outlierTable[,j])$p.value,10)
      }

    }
  }
  coocurrence
}

outlierSummaryByClass <- function(motifOutliers,k,arborType=NA,cellType=NA,brainRegion=NA,minLength=NaN,maxLength=NaN){
  subData = getSubset(motifOutliers,arborType,cellType,brainRegion,minLength,maxLength)
  result <- mean(subData[,3:dim(subData)[2]])
  result
}

testOutliersByClass <- function(motifOutliers1,motifOutliers2,k,test=wilcox.test,order="LTS",arborType=NA,cellType=NA,brainRegion=NA,minLength=NaN,maxLength=NaN,randomSet=0,randomK=0,freq=T){
  subData1 = getSubset(motifOutliers1,arborType,cellType,brainRegion,minLength,maxLength)
  subData2 = getSubset(motifOutliers2,arborType,cellType,brainRegion,minLength,maxLength)
  test(subData1$OutlierFraction,subData2$OutlierFraction)
}

outlierRatioByClass <- function(motifOutliers,k,order="LTS",arborType=NA,cellType=NA,brainRegion=NA,minLength=NaN,maxLength=NaN,randomSet=0,randomK=0,freq=T,breaks="Sturges"){
  titleText <- paste("Outlier Ratio of ",order," ",k,"-mers",sep="")
  titleText <- getSubsetTitle(titleText,arborType,cellType,brainRegion,minLength,maxLength,randomSet,randomK)
  subData = getSubset(motifOutliers,arborType,cellType,brainRegion,minLength,maxLength)
  x11()
  hist(subData$OutlierFraction,main=titleText,xlab="Kmer Outlier Ratio",freq=freq,breaks=breaks)
}

getSubsetTitle <- function(titletxt="",arborType=NA,cellType=NA,brainRegion=NA,minLength=NaN,maxLength=NaN,randomSet=0,randomK=0){
  if (randomSet > 0){
    titletxt = paste(titletxt," (RandomSet",randomSet," K",randomK,")",sep="")
  }
  if (!is.na(arborType) || !is.na(cellType) || !is.na(brainRegion)){
    titletxt = paste(titletxt,"\n",sep="")
    divider = ""
    if (!is.na(arborType)){
      titletxt = paste(titletxt,"Arbor Type: ",arborType,sep="")
      divider = "; "
    }
    if (!is.na(cellType)){
      titletxt = paste(titletxt,divider,"Cell Type: ",cellType,sep="")
      divider = "; "
    }
    if (!is.na(brainRegion)){
      titletxt = paste(titletxt,divider,"Brain Region: ",brainRegion,sep="")
      divider = "; "
    }
  }
  else {
    titletxt = paste(titletxt,"\nAll neurites",sep="")
  }
  if (!is.nan(minLength) && !is.nan(maxLength)){
    titletxt = paste(titletxt,"\nLengths ",minLength," through ",maxLength,sep="")
  }
  else if (!is.nan(minLength) && is.nan(maxLength)){
    titletxt = paste(titletxt,"\nLengths equal to or greater than ",minLength,sep="")
  }
  else if (is.nan(minLength) && !is.nan(maxLength)){
    titletxt = paste(titletxt,"\nLengths less than or equal to ",maxLength,sep="")
  }
  titletxt
}

getSubsetSequences <- function(mydata,arborType=NA,cellType=NA,brainRegion=NA,minLength=NaN,maxLength=NaN){
  subData <- mydata
  if (!is.na(arborType)){
    if (arborType == "Dendrite" || arborType == "BasalDendrite"){
      sequences <- as.character(subset(metaData,ArborType=="Dendrite"|ArborType=="BasalDendrite")$Sequence)
    }
    else{
      sequences <- as.character(subset(metaData,ArborType==arborType)$Sequence)
    }
    subData <- subset(subData,Sequence%in%sequences)
  }
  if (!is.na(cellType)){
    sequences <- as.character(subset(metaData,CellType==cellType)$Sequence)
    subData <- subset(subData,Sequence%in%sequences)
  }
  if (!is.na(brainRegion)){
    sequences <- as.character(subset(metaData,BrainRegion==brainRegion)$Sequence)
    subData <- subset(subData,Sequence%in%sequences)
  }
  if (!is.nan(minLength)){
    sequences <- as.character(subset(metaData,SeqLen>=minLength)$Sequence)
    subData <- subset(subData,Sequence%in%sequences)
  }
  if (!is.nan(maxLength)){
    sequences <- as.character(subset(metaData,SeqLen<=maxLength)$Sequence)
    subData <- subset(subData,Sequence%in%sequences)
  }
  subData$Sequence
}
getSubset <- function(mydata,arborType=NA,cellType=NA,brainRegion=NA,minLength=NaN,maxLength=NaN){
  subData <- mydata
  if (!is.na(arborType)){
    if (arborType == "Dendrite" || arborType == "BasalDendrite"){
      sequences <- as.character(subset(metaData,ArborType=="Dendrite"|ArborType=="BasalDendrite")$Sequence)
    }
    else{
      sequences <- as.character(subset(metaData,ArborType==arborType)$Sequence)
    }
    subData <- subset(subData,Sequence%in%sequences)
  }
  if (!is.na(cellType)){
    sequences <- as.character(subset(metaData,CellType==cellType)$Sequence)
    subData <- subset(subData,Sequence%in%sequences)
  }
  if (!is.na(brainRegion)){
    sequences <- as.character(subset(metaData,BrainRegion==brainRegion)$Sequence)
    subData <- subset(subData,Sequence%in%sequences)
  }
  if (!is.nan(minLength)){
    sequences <- as.character(subset(metaData,SeqLen>=minLength)$Sequence)
    subData <- subset(subData,Sequence%in%sequences)
  }
  if (!is.nan(maxLength)){
    sequences <- as.character(subset(metaData,SeqLen<=maxLength)$Sequence)
    subData <- subset(subData,Sequence%in%sequences)
  }
  subData
}

runStatTest <- function(dataSet1,dataSet2,test=c("wilcoxin","binomial","poisson"),alternative=c("two.sided", "less", "greater")){
  if (test == "wilcoxin"){
    testResult <- wilcox.test(dataSet1,dataSet2,paired=T,alternative=alternative)
  }
  else if (test == "binomial"){
    testResult <- binom.test(sum(dataSet1),length(dataSet1),mean(dataSet2),alternative=alternative)
  }
  else if (test == "poisson"){
    testResult <- poisson.test(c(sum(dataSet1),sum(dataSet2)),alternative=alternative)
  }
  testResult
}

