
angleBetweenVectors <- function(a,b){
  return(acos( sum(a*b) / ( sqrt(sum(a * a)) * sqrt(sum(b * b)) ) ))
}

crossProduct <- function(a,b){
  result <- t(c(0,0,0))
  result[1] <- a[2] * b[3] - a[3] * b[2] 
  result[2] <- a[3] * b[1] - a[1] * b[3] 
  result[3] <- a[1] * b[2] - a[2] * b[1] 
  return(result)
}

orthogonalVector <- function(a,b){
  a <- a/sqrt(sum(a^2))
  b <- b/sqrt(sum(b^2))
  result <- t(c(0,0,0))
  result[1] <- a[2] * b[3] - a[3] * b[2] 
  result[2] <- a[3] * b[1] - a[1] * b[3] 
  result[3] <- a[1] * b[2] - a[2] * b[1] 
  return(result)
}

normOrthogonalVector <- function(a,b){
  orth <- orthogonalVector(a,b)
  return(orth/sqrt(sum(orth^2)))
}

rotationMatrix <- function(vectorA,vectorB){
  orth <- normOrthogonalVector(vectorA,vectorB)
  x <- orth[1]
  y <- orth[2]
  z <- orth[3]
  theta <- angleBetweenVectors(vectorA,vectorB)
  rotMat <- matrix(nrow=3,ncol=3)
  cosTheta <- cos(theta)
  sinTheta <- sin(theta)
  
  rotMat[1,1] <- cosTheta + x^2*(1-cosTheta)
  rotMat[1,2] <- x*y*(1-cosTheta) - z*sinTheta
  rotMat[1,3] <- x*z*(1-cosTheta) + y*sinTheta
  
  rotMat[2,1] <- y*x*(1-cosTheta) + z^2*sinTheta
  rotMat[2,2] <- cosTheta + y^2*(1-cosTheta)
  rotMat[2,3] <- y*z*(1-cosTheta) - x*sinTheta

  rotMat[3,1] <- z*x*(1-cosTheta) - y*sinTheta
  rotMat[3,2] <- z*y*(1-cosTheta) + x*sinTheta
  rotMat[3,3] <- cosTheta + z^2*(1-cosTheta)
  
  return(rotMat)
}

reflectPoint <- function(point,slope,intercept){
  x <- point[1]
  y <- point[2]
  orthSlope <- -1/slope
  orthIntercept <- y-x*orthSlope
  nearestPointOnLineX <- slope*(orthIntercept - intercept)/(slope^2 + 1)
  nearestPointOnLineY <- slope*nearestPointOnLineX+intercept
  reflectedPoint <- c(2*nearestPointOnLineX - x,2*nearestPointOnLineY - y)
  return(reflectedPoint)
}
